// Axis.cpp: implementation of the CAxis class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "aldas.h"
#include "Axis.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAxis::CAxis (const char* c, int n, float* d, float* e, float* chisq){

/*#ifdef _DEBUG
	oldMemState = new CMemoryState;
	newMemState = new CMemoryState;
	diffMemState = new CMemoryState;
	oldMemState->Checkpoint();
#endif*/

	if (c!=NULL) {
		label=c;
	} else {
		label.Empty();
	}
	len=n;
	data=d;
	error=e;
	chi2=chisq;
	type=LinAxis;
	FindMinMax();
	FindScale(min, max, low, high, nticks, mticks);

	numBins = 100;
	binMaxValue = 0;
	histArray=NULL;
	//MakeHistogram();
}

CAxis::~CAxis() {
	if (histArray) delete [] histArray;

/*#ifdef _DEBUG
	newMemState->Checkpoint();
	if (diffMemState->Difference( *oldMemState, *newMemState ) ) {
		TRACE( "Memory leaked in CAxis!\n" );
		//diffMemState->DumpAllObjectsSince();
		diffMemState->DumpStatistics();
	} else {
		TRACE( "NO Memory leaked in CAxis!\n" );
	}
	delete oldMemState;
	delete newMemState;
	delete diffMemState;
#endif*/
}


void CAxis::FindMinMax()
{
	int i;

	min=0.0;
	max=1.0;
	if (data==NULL) return;

	if (type==LinAxis){
		if (chi2) {
			for (i=0; (i<len)&&(chi2[i]<0); i++); // advance to valid data
		} else {
			i=0;
		}
		if (i==len) return; // no valid data
		min=data[i];
		max=data[i];

		if (error==NULL) {
			for (i=0; i<len; i++){
				if ((chi2==NULL)||(chi2[i]>=0)) { // data valid
					if (min>data[i]) min=data[i];
					if	(max<data[i]) max=data[i];
				}
			}
		} else {
			for (i=0; i<len; i++){
				if ((chi2==NULL)||(chi2[i]>=0)) { // data valid
					if (min>data[i]-error[i]) min=data[i]-error[i];
					if (max<data[i]+error[i]) max=data[i]+error[i];
				}
			}
		} 
	}
	if (type==LogAxis){
		if (chi2) {
			for (i=0; (i<len)&&(chi2[i]<0)&&(data[i]>0); i++); // advance to valid data
		} else {
			i=0;
		}
		if (i==len) return; // no valid data
		min=data[i];
		max=data[i];

		if (error==NULL) {
			for (i=0; i<len; i++){
				if ((chi2==NULL)||(chi2[i]>=0)) { // data valid
					if ((min>data[i])&&(data[i]>0)) min=data[i];
					if	((max<data[i])&&(data[i]>0)) max=data[i];
				}
			}
		} else {
			for (i=0; i<len; i++){
				if ((chi2==NULL)||(chi2[i]>=0)) { // data valid
					if ((min>data[i]-error[i])&&(data[i]-error[i]>0)) min=data[i]-error[i];
					if ((max<data[i]+error[i])&&(data[i]+error[i]>0)) max=data[i]+error[i];
				}
			}
		} 
	}
}

int CAxis::MapValue(float x, float x1, float x2)
// maps world coord. x to screen coord. in the range of [x1, x2]
{
	int i;
	if (x1<0) x1=ymin; // default
	if (x2<0) x2=ymax; // default
	if (type==LinAxis)	i= int(x1+(x-low)/(high-low)*(x2-x1));
	if (type==LogAxis){
		if (x>0.0) {
			i= int(x1+log(x/low)/log(high/low)*(x2-x1));
		} else {
			i=int(x1);
		}
	}
	return i;
}

  

void CAxis::FindScale (float min, float max, float& lower, float& upper, int& nticks, int&mticks) {
	float range, newrange, a, r;
	int i, k;
	int y[12]      = {10, 12, 15, 16, 20, 25, 30, 40, 50, 60, 80, 100};
   int nTicks[12] = {10, 12, 15,  8, 10,  5,  6,  8, 10,  6,  8, 10};
   int mTicks[12] = { 5,  6,  3,  4,  5,  5,  6,  8,  5,  6,  8,  5};

	if (type==LinAxis) {
		range=max-min;
		a=log10(range);
		k=floor(a);
		r=10.0*exp((a-k)*log(10.0)); // r=10 .. 99
		for (i=0; ((i<12)&&(r>=y[i])); i++);
		newrange=y[i]*exp((k-1)*log(10.0));
		nticks=nTicks[i];
		mticks=mTicks[i];
		lower=floor(min/(newrange/nticks))*newrange/float(nticks);
		upper= lower + newrange;
		while (upper<max) { // expand newrange...
//			TRACE ("Old Scale: %f ... %f, min=%f, max=%f\n", lower, upper, min, max);
			i++;
			if (i>11) {
				i=0;
				k++;
			}
			newrange=y[i]*exp((k-1)*log(10.0));
			nticks=nTicks[i];
			mticks=mTicks[i];
			lower= floor(min/(newrange/nticks))*newrange/float(nticks);
			upper= lower + newrange;
//			TRACE ("New Scale: %f ... %f, min=%f, max=%f\n", lower, upper, min, max);
		}
	}
	if (type==LogAxis) {
		if (min>0){	
			a=log10(min);
		} else {
			a=-3.0;
			TRACE ("CAxis::FindScale: min<0 for log scale\n");
		}
		k=floor(a);
		lower=10.0*exp((k-1)*log(10.0));
		if (max>0){	
			a=log10(max)-0.001;
		} else {
			a=-3.0;
			TRACE ("CAxis::FindScale: max<0 for log scale\n");
		}
		k=floor(a);
		upper=10.0*exp(k*log(10.0));
		nticks=int(floor(log10(upper/lower)))+1;
		mticks=nticks;
	}
	return;
}


void CAxis::SetLabel (const CString& l)
{
	label=l;
}

int CAxis::GetFormat()
{
	float interval=(high-low)/mticks; // difference between 2 labeled tick marks
	int i =-int(log10(interval)) + 1;
	if (interval>1.0) i=0; // no decimals needed
	if (i>4) i=4;
	return i;
}

void CAxis::DrawData(CDC* pDC, CRect rect)
{
	CString buf;
	int i;
	int ix, iy;
	CSize size;
	int symbsize;

	CPen *oldPen, newPen; // newPen is destroyed automatically.
	float sx; // scale for the x axis
	CString form;

	DrawYScale(pDC, rect);
	symbsize=ticksize/2; //size in pixel
	size=pDC->GetTextExtent("Channel");
	pDC->TextOut(int(xmax)-2*size.cx, 
								rect.Height()-int(1.25*size.cy)+rect.top, "Channel");
	pDC->TextOut(3+rect.left, int(0.05*size.cy)+rect.top, GetLabel());

	//////////////////// X Axis //////////////////		
	for (i=0; i<len/16; i++){
		ix=int(xmin+(xmax-xmin)/(len/16)*i);
		pDC->MoveTo(ix, ymin);
		pDC->LineTo(ix, ymin+ticksize/2+1);
	}		
	buf.Format("%i", 0);
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(int(xmin-size.cx/2), int(ymin+0.25*size.cy), buf);
	buf.Format("%i", len);
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(int(xmax-size.cx/2), int(ymin+0.25*size.cy), buf);
   ix=int(xmin);
	pDC->MoveTo(ix, ymin);
	pDC->LineTo(ix, ymin+ticksize+1);
   ix=int(xmax)+1;
	pDC->MoveTo(ix, ymin);
	pDC->LineTo(ix, ymin+ticksize+1);



	/////////////////// draw data points //////////////////
	newPen.CreatePen(PS_SOLID, 1, RGB(0, 0, 255));
	oldPen=pDC->SelectObject(&newPen);
	sx=(xmax-xmin)/float(len);
	if ((data==NULL)||(len<1)) {
		size=pDC->GetTextExtent("- Empty -");
		pDC->TextOut(int(xmin+(xmax-size.cx)/2), int(ymin+(ymax-size.cy)/2), "- Empty -");
		pDC->SelectObject(oldPen);
		return;
	}
	for (i=0; i<len; i++){
		if ((chi2==NULL)||((chi2[i]>=0)&&((type==LinAxis)||(data[i]>0)))) { // data valid
			ix=int(sx*i+xmin+0.5);
			if (error) { // have errors on y
				iy=MapYValue(data[i]-error[i]);
				pDC->MoveTo(ix-symbsize, iy);
				pDC->LineTo(ix+symbsize+1, iy);
				pDC->MoveTo(ix, iy);
				iy=MapYValue(data[i]+error[i]);
				pDC->LineTo(ix, iy);
				pDC->MoveTo(ix-symbsize, iy);
				pDC->LineTo(ix+symbsize+1, iy);
			}
			iy=MapYValue(data[i]);
			pDC->Ellipse(ix-symbsize, iy+symbsize, ix+symbsize+1, iy-symbsize);
		}
	}
	pDC->SelectObject(oldPen);
}


void CAxis::MakeHistogram() {
	int i;
	binMaxValue = 0;
	float binSize = (max - min)/numBins;

	if (histArray) delete[] histArray;
	histArray = new int[numBins];

	for (i=0; i<numBins; i++)	histArray[i] = 0;
	int binCounter = 0;
	for (i=0; i<len; i++) {
		if ((chi2==NULL)||(chi2[i]>=0)) { // data valid
			binCounter = int((data[i]-min)/binSize);
			if ((binCounter>=0)&&(binCounter<numBins)) {
				histArray[binCounter] ++;
				if (histArray[binCounter]>binMaxValue) binMaxValue = histArray[binCounter];
			}
		}
	}
}


void CAxis::DrawHistogram(CDC* pDC, CRect rect) 
{ // copied from CCollectDataView
	CString buf, form;
	int i;
	int ix, iy, y;
	float x;
	CSize size;
	float upper, lower;
	int nTicks, mTicks, ticksize;
	CPen *oldPen, newPen; // newPen is destroyed automatically.
	float sx, sy; // scales

	int wid=rect.Width();
	int hei=rect.Height();
	ticksize = 10*(wid+hei)/1200;


	if (!histArray) MakeHistogram();


	// get number of tick marks:
	FindScale (0, float(binMaxValue), lower, upper, nTicks, mTicks); 
	if (nTicks>upper) nTicks=upper; // no fractional tick marks
	if (mTicks>upper) mTicks=upper; // no fractional tick marks

	size=pDC->GetTextExtent("123gyj");
	// histogram size in screen coord.:
	ymin=rect.top+float(hei-1.5*size.cy); // 1.5 lines on bottom
	ymax=rect.top+float(1.5*size.cy);     // 1.5 lines on top
	xmin=0.0;

	//////////////////// y axis /////////////////////
	for (i=0; i<=mTicks; i++){
		buf.Format(" %i ", int(upper*i)/mTicks);
		size=pDC->GetTextExtent(buf);
		if (xmin<size.cx) xmin=size.cx;
		iy=int(ymin+(ymax-ymin)/float(mTicks)*i-0.5*size.cy);
		pDC->TextOut(rect.left, iy, buf);
	}
	xmin=xmin+ticksize+rect.left;
	xmax=float(wid*0.975)+rect.left;

	for (i=0; i<=nTicks; i++){
		iy=int(ymin+(ymax-ymin)/float(nTicks)*i);
		pDC->MoveTo((int)xmin, iy);
		pDC->LineTo((int)xmin-ticksize/2, iy);
	}
	for (i=0; i<=mTicks; i++){
		iy=int(ymin+(ymax-ymin)/float(mTicks)*i);
		pDC->MoveTo((int)xmin, iy);
		pDC->LineTo((int)xmin-ticksize, iy);
	}
	/////////////////// Frame /////////////////////
	pDC->MoveTo((int)xmin, (int)ymin);
	pDC->LineTo((int)xmax+1, (int)ymin);
	pDC->LineTo((int)xmax+1, (int)ymax);
	pDC->LineTo((int)xmin, (int)ymax);
	pDC->LineTo((int)xmin, (int)ymin);

	size=pDC->GetTextExtent(label);
	pDC->TextOut(int(xmax)-2*size.cx, 
								hei-int(1.25*size.cy)+rect.top, label);
	//pDC->TextOut(3+rect.left, int(0.05*size.cy)+rect.top, "Counts");

	//////////////////// X Axis //////////////////		
	FindScale (min, max, lower, upper, nTicks, mTicks); 
	form.Format("%1i", GetFormat());
	form = "%7." + form +"f";

	for (i=0; i<=nTicks; i++){
		ix=int(xmin+(xmax-xmin)/float(nTicks)*i);
		pDC->MoveTo(ix, (int)ymin);
		pDC->LineTo(ix, (int)ymin+ticksize/2);
	}
	if (mTicks>nTicks) mTicks=nTicks;
	for (i=0; i<=mTicks; i++){
		ix=int(xmin+(xmax-xmin)/float(mTicks)*i);
		pDC->MoveTo(ix, (int)ymin);
		pDC->LineTo(ix, (int)ymin+ticksize);
		x=(upper-lower)*i/float(mTicks)+lower;
		buf.Format(form, x);
		size=pDC->GetTextExtent(buf);
		pDC->TextOut(ix-size.cx/2, ymin+int(0.25*size.cy), buf);
	}


	/////////////////// draw data /////////////////////////
	FindScale (0, float(binMaxValue), lower, upper, nTicks, mTicks); // restore lower, upper
	newPen.CreatePen(PS_SOLID, 1, RGB(128, 0, 0));
	oldPen=pDC->SelectObject(&newPen);
	sy=-(ymax-ymin)/upper;
	sx=(xmax-xmin)/float(numBins);
	for (i=0; i<numBins; i++){
		y=int(sy*histArray[i]+0.5);  
		ix=int(sx*i+xmin+0.5);
		pDC->Rectangle(ix, int(ymin)-y, int(sx*(i+1)+xmin+1.5), int(ymin)); 
	}
	pDC->SelectObject(oldPen);
}


int CAxis::GetChannelNumber (CPoint point)
{
	if ((point.x>=xmin)&&(point.x<=xmax)) {
		return int(len*(point.x-xmin-0.5)/(xmax-xmin));
	} else {
		return -1;
	}
}


int CAxis::GetChannelNumberY (CPoint point)
{
	if ((point.y<=ymin)&&(point.y>=ymax)) {
		return int(len*(-point.y+ymin-0.5)/(ymin-ymax));
	} else {
		return -1;
	}
}

BOOL CAxis::CursorInsideGraphRect (CPoint point) {
//	TRACE ("%i %i %i %i\n", xmin, xmax, ymin, ymax);
//	TRACE ("%i %i\n", point.x, point.y);
	if ((point.x>=xmin)&&(point.x<=xmax)&&(point.y<=ymin)&&(point.y>=ymax))
		return TRUE;
	else
		return FALSE;
}


float* CAxis::GetChiSquare()
{
	return chi2;
}

void CAxis::PlotVersus(CDC* pDC, CRect rect, CAxis* xAxis)
{// plots this axis versus the xAxis 
	CString buf;
	int i;
	int ix, iy;
	CSize size;
	int symbsize;
	float upper, lower, x;

	CPen *oldPen, newPen; // newPen is destroyed automatically.
	CString form;

	// Make sure the pointer chi2 has been initialized.
	// If *chi2 is < 0 then don't plot the data.
	if (chi2) {
		if (*chi2<0) {
			size=pDC->GetTextExtent("-Error in fitting routine-");
			pDC->TextOut(int(xmin+(xmax-size.cx)/2), int(ymin+(ymax-size.cy)/2), 
				"-Error in fitting routine-");
			pDC->SelectObject(oldPen);
			return;
		}
	}

	DrawYScale(pDC, rect);
	xAxis->xmax=xmax;
	xAxis->xmin=xmin;
	xAxis->ymax=ymax;
	xAxis->ymin=ymin;


	symbsize=ticksize/2; //size in pixel

	size=pDC->GetTextExtent(xAxis->GetLabel());
	pDC->TextOut(int(xmax)-size.cx-10, 
 					rect.bottom-int(0.25*size.cy)-2, xAxis->GetLabel());
	pDC->TextOut(3+rect.left, int(0.05*size.cy)+rect.top, GetLabel());

	//////////////////// X Axis //////////////////		
	upper=xAxis->GetHigh(); // axis range
	lower=xAxis->GetLow();
	form.Format("%1i", xAxis->GetFormat());
	form = "%7." + form +"f";
	for (i=0; i<=xAxis->GetmTicks(); i++){
		x=(upper-lower)*i/float(xAxis->GetmTicks())+lower;
		buf.Format(form, x);
		size=pDC->GetTextExtent(buf);
		ix=xAxis->MapXValue(x, xmin, xmax);
		pDC->TextOut(ix-size.cx/2, rect.Height()-int(1.25*size.cy)+rect.top, buf);
		pDC->MoveTo(ix, (int)ymin);
		pDC->LineTo(ix, (int)ymin+ticksize);
	}

	for (i=0; i<=xAxis->GetnTicks(); i++){
		ix=int(xmin+(xmax-xmin)/float(xAxis->GetnTicks())*i);
		pDC->MoveTo(ix, (int)ymin);
		pDC->LineTo(ix, (int)ymin+ticksize/2);
	}

	/////////////////// draw data points //////////////////
	newPen.CreatePen(PS_SOLID, 1, RGB(0, 128, 0));
	oldPen=pDC->SelectObject(&newPen);
	if ((data==NULL)||(len<1)||(xAxis->GetData()==NULL)||(xAxis->GetLength()<1)) {
		size=pDC->GetTextExtent("- Empty -");
		pDC->TextOut(int(xmin+(xmax-size.cx)/2), int(ymin+(ymax-size.cy)/2), "- Empty -");
		pDC->SelectObject(oldPen);
		return;
	}
	for (i=0; i<len; i++){
		ix=xAxis->MapXValue(xAxis->GetData()[i], xmin, xmax);
		if (error) { // have errors on y
			iy=MapYValue(data[i]-error[i]);
			pDC->MoveTo(ix-symbsize, iy);
			pDC->LineTo(ix+symbsize+1, iy);
			pDC->MoveTo(ix, iy);
			iy=MapYValue(data[i]+error[i]);
			pDC->LineTo(ix, iy);
			pDC->MoveTo(ix-symbsize, iy);
			pDC->LineTo(ix+symbsize+1, iy);
		}
		iy=MapYValue(data[i]);
		if (xAxis->error) { // have errors on x
			ix=xAxis->MapXValue(xAxis->data[i]-xAxis->error[i], xmin, xmax);
			pDC->MoveTo(ix, iy-symbsize);
			pDC->LineTo(ix, iy+symbsize+1);
			pDC->MoveTo(ix, iy);
			ix=xAxis->MapXValue(xAxis->data[i]+xAxis->error[i], xmin, xmax);
			pDC->LineTo(ix, iy);
			pDC->MoveTo(ix, iy-symbsize);
			pDC->LineTo(ix, iy+symbsize+1);
		}
		ix=xAxis->MapXValue(xAxis->GetData()[i], xmin, xmax);
		pDC->Ellipse(ix-symbsize, iy+symbsize, ix+symbsize+1, iy-symbsize);
	}
	pDC->SelectObject(oldPen);
}

void CAxis::SetType(AxisType t)
{
	type=t;
	FindMinMax();
	FindScale(min, max, low, high, nticks, mticks);
}

void CAxis::DrawYScale(CDC* pDC, CRect rect)
{
	CString buf;
	int i;
	int iy, wid, hei;
	CSize size;
	int nTicks, mTicks;
	float upper, lower, y;
	CString form;

	wid=rect.Width();
	hei=rect.Height();
	ticksize = (wid+hei)/120;

	nTicks=GetnTicks(); 
	mTicks=GetmTicks(); 
	if (mTicks>20) mTicks=nTicks;
	upper=GetHigh(); // axis range
	lower=GetLow();
	//if (nTicks>upper) nTicks=upper; // no fractional tick marks
	//if (mTicks>upper) mTicks=upper; // no fractional tick marks

	size=pDC->GetTextExtent("123gyj");
	// graph size in screen coord.:
	ymin=rect.top + hei-int(1.5*size.cy); // 1.5 lines on bottom
	ymax=rect.top + int(1.5*size.cy);     // 1.5 lines on top
	xmin=0.0;

	//////////////////// y axis /////////////////////
	if (type==LinAxis) {
		form.Format("%1i", GetFormat());
		form = " %7." + form +"f ";
		for (i=0; i<=mTicks; i++){
			y=(upper-lower)*i/float(mTicks)+lower;
			buf.Format(form, y);
			size=pDC->GetTextExtent(buf);
			if (xmin<size.cx) xmin=size.cx;
			iy=MapYValue(y) -size.cy/2;
			pDC->TextOut(rect.left, iy, buf);
		}
		xmin=xmin+ticksize+rect.left;
		xmax=wid*0.975+rect.left;

		for (i=0; i<=nTicks; i++){
			iy=int(ymin+(ymax-ymin)/float(nTicks)*i);
			pDC->MoveTo(xmin, iy);
			pDC->LineTo(xmin-ticksize/2, iy);
		}	
		for (i=0; i<=mTicks; i++){
			iy=int(ymin+(ymax-ymin)/float(mTicks)*i);
			pDC->MoveTo(xmin, iy);
			pDC->LineTo(xmin-ticksize, iy);
		}
	}
	if (type==LogAxis) {
		y=lower;
		for (i=0; (i<=mTicks)&&(y<=upper); i++){
			buf.Format("10^%2i", int(floor(0.5+log10(y))));
			size=pDC->GetTextExtent(buf);
			if (xmin<size.cx) xmin=size.cx;
			iy=MapYValue(y) -size.cy/2;
			pDC->TextOut(rect.left, iy, buf);
			y*=10.0;
		}
		xmin=xmin+ticksize+rect.left;
		xmax=wid*0.975+rect.left;

		y=lower;
		for (i=0; i<mTicks; i++){
			iy=MapYValue(y);
			pDC->MoveTo(xmin, iy);
			pDC->LineTo(xmin-ticksize, iy);
			y*=10.0;
		}
	}
	/////////////////// Frame /////////////////////
	pDC->MoveTo((int)xmin, (int)ymin);
	pDC->LineTo((int)xmax+1, (int)ymin);
	pDC->LineTo((int)xmax+1, (int)ymax);
	pDC->LineTo((int)xmin, (int)ymax);
	pDC->LineTo((int)xmin, (int)ymin);

}

int CAxis::MapXValue(float x, float x1, float x2)
// maps world coord. x to screen coord. in the range of [xmin, xmax]
{
	int i;
	if (x1<0) x1=xmin; // default
	if (x2<0) x2=xmax; // default
	if (type==LinAxis)	i= int(x1+(x-low)/(high-low)*(x2-x1));
	if (type==LogAxis){
		if (x>0.0) {
			i= int(x1+log(x/low)/log(high/low)*(x2-x1));
		} else {
			i=int(x1);
		}
	}
	return i;
}

  

int CAxis::MapYValue(float y)
// maps world coord. y to screen coord. in the range of [ymin, ymax]
{
	int i;
	if (type==LinAxis)	i= int(ymin+(y-low)/(high-low)*(ymax-ymin));
	if (type==LogAxis){
		if (y>0.0) {
			i= int(ymin+log(y/low)/log(high/low)*(ymax-ymin));
		} else {
			i=int(ymin);
		}
	}
	return i;
}

float CAxis::Mean(int start, int stop, float& sigma)
{
	float sum=0, sum2=0;
	int i, N=0;
	for (i=start; i<stop; i++){
		if ((chi2==NULL)||(chi2[i]>=0)) { // data valid
			sum += data[i];
			sum2+= data[i]*data[i];
			N++;
		}
	}
	if (N>0) {
		sum = sum/float(N);
		sigma = sqrt(sum2/float(N) - sum*sum);
	} 	else {
		sum = 0.0;
		sigma = 0.0;
	}
	return sum;
}

