#include "stdafx.h"
#include <math.h>
#include "FitFunction.h"
#include "CurveFitter.h"
#include "Matrix.h"
#include "Vector.h"


static float sqrarg;
#define SQR(a) (sqrarg=(a), sqrarg*sqrarg)
#define ITMAX 100
#define EPS 3.0e-7
#define SWAP(a,b) {double temp=(a);(a)=(b);(b)=temp;}

CCurveFitter::CCurveFitter() {
}

CCurveFitter::~CCurveFitter() {
}

void CCurveFitter::PolInt(float xa[], float ya[], int n, float x, float *y, float *dy) {
	int i, m, ns=1;
	float den, dif, dift, ho, hp, w;
	float *c, *d;

	dif=fabs(x-xa[1]);
	c = new float [n];
	d= new float [n];
	for (i=1; i<=n;i++) {
		if ((dift=fabs(x-xa[i])) < dif) {
			ns=i;
			dif=dift;
		}
		c[i]=ya[i];
		d[i]=ya[i];
	}
	*y=ya[ns--];
	for (m=1;m<n;m++) {
		for (i=1;i<=n-m;i++) {
			ho=xa[i]-x;
			hp=xa[i+m]-x;
			w=c[i+1]-d[i];
			if ((den=ho-hp) == 0.0) 
				AfxMessageBox("Error in routine PolInt", MB_ICONEXCLAMATION|MB_OK);
			den=w/den;
			d[i]=hp*den;
			c[i]=ho*den;
		}
		*y+= (*dy=(2*ns < (n-m) ? c[ns+1] : d[ns--]));
	}
	delete [] c;
	delete [] d;
}


void CCurveFitter::LsqFit(float * x, float * y, float ndata,  float weight[], int mwt, 
									 float *a, float *b, float *siga, float *sigb, float *chi2, 
									 float *q) {
	int i;
	float wt, t, sxoss, sx=0.0, sy=0.0, st2=0.0, ss, sigdat;
//	float gammq();

	*b = 0.0;
	if (mwt) {
		ss=0.0;
		for (i=0; i<ndata; i++) {
			wt=SQR(weight[i]);
			ss += wt;
			sx += x[i]*wt;
			sy += y[i]*wt;
		}
	} else {
		for (i=0; i<ndata; i++) {
			sx += x[i];
				sy += y[i];
		}
		ss=ndata;
	}
	sxoss = sx/ss;
	if (mwt) {
		for (i=0; i<ndata; i++) {
			t=(x[i]-sxoss)*weight[i];
			st2 += t*t;
			*b += t*y[i]* weight[i];
		}
	} else {
		for (i=0; i<ndata; i++) {
			t=x[i]-sxoss;
			st2 += t*t;
			*b += t*y[i];
		}
	}
	*b /= st2;
	*a = (sy-sx*(*b))/ss;
	*siga=sqrt((1.0+sx*sx/(ss*st2))/ss);
	*sigb=sqrt(1.0/st2);
	*chi2=0.0;
	if (mwt==0) {
		for (i=0; i<ndata; i++)
			*chi2 += SQR (y[i]-(*a)-(*b)*x[i]);
		*q=1.0;
		sigdat=sqrt((*chi2)/(ndata-2));
		*siga *=sigdat;
		*sigb *=sigdat;
	} else {
		for (i=0; i<ndata; i++)
			*chi2 += SQR((y[i]-(*a)-(*b)*x[i])*weight[i]);
//		*q=GammQ(0.5*(ndata-2),0.2*(*chi2));
		*q = 0;
	}
}

int CCurveFitter::FitErf (float *iy, float *x, int n, int loops, float *pars, 
								  float *errs, float *Rchisq, int mode)
/* attempts to least-squares fit an error function (integral of gaussian) to the
given data (from 0 to loops).  

Data are assumed to come from a binomial distribution (efficiency plot).
X values should be monotonicly increasing, Y values should tend to go from
higher at low X to lower at large X. 

Possible result codes:

1:  Dead.  All data is less than 2%
2:  Weak.  Couldn't fit because all data less than 25%.
3:  Stuck.  Couldn't fit because all data greater than 80%.
4:  Stuck.  Data pegged at one occupancy.
5:  Range.  Data was either below 0 or above 100.
6:  Data.  Too few data points in region of interest to make fit.
7:  Curve Fit failed (probably due to a singular matrix, data may not be believable)
8:  Chisq per degree of freedom too large to believe fit. */

{
	int     i, nltot = 0;
	float  *y;
	float  *sig, *a;
	Tmatrix<float> covar(2, 2);
	Tmatrix<float> alpha(2, 2);
	float  alamda;
	float  ymax=0.;
	int     lista[3], nmin = n-1, nmax = n, nc;
	float  sx = 0, sxx = 0, sxy = 0, sy = 0;
	float  mu, lxsq, fxsq, xsq, dxsq, dx=50.;
	int     dead=1, stuck=1, weak=1, range=0, nstable=0;
 
 	y = vector(0,n);


	for ( i = 0; i < n; i++ ) 
	{
		if ( mode == 0 )
			y[i]  = 100.0 * iy[i] / ( (float) loops );
		else
			y[i]  = 100.0 - 100.0 * iy[i] / ( (float) loops );

		if ( y[i] > 110. || y[i] < -10.0)  range = 1;
		if ( y[i] < 0.0 )   y[i]  = 0.0;
		if ( y[i] > 100.0 ) y[i]  = 100.0;
		if ( y[i] > 2.0 )   dead  = 0;
		if ( y[i] > 25.0 )  weak  = 0;
		if ( y[i] < 80.0 )  stuck = 0;
	}


	if ( dead )   return ( 1 );
	if ( weak )   return ( 2 );
	if ( stuck )  return ( 3 );
	if ( range )  return ( 5 );
	
	stuck = 1;
	for ( i = n-1; i > 0; i-- ) 
	{
		if ( y[i] < 2.)   nmax = i+1;
		if ( y[i] > ymax) ymax = y[i];
		if ( nmin == i && (y[i] > ymax - 10.) && (y[i]<=99.)) nmin = i-1;
		if ( fabs (y[i] - y[i-1]) > 5.0 )  stuck = 0;
		if ( y[i]>10. && y[i] < 90.)  {
			
			if ( fabs( y[i] - 50.0 ) < dx ) { 
				dx = fabs ( y[i] - 50.0 ); 
				mu = x[i]; 
			}

			sx  += x[i];
			sy  += y[i];
			sxy += x[i] * y[i];
			sxx += x[i] * x[i];
		}
	}
	

	if ( nmin==0 && y[0] > ymax - 10.) { 
		nc = nmax; 
		nmin = -1; 
	}
	else 
		nc = nmax - nmin - 1;
		
	
	a    = vector(1,2);
	a[1] = mu;
	a[2] = 15.; 
	if ( stuck )   return ( 4 );
	if ( nc < 3 )  return ( 6 );



	// least-square fit slope initially
	
 	sig = vector(1,nc);
	x   = &(x[nmin]); y = &(y[nmin]);
	for ( i = 1; i <= nc; i++ )
		 sig[i] = sqrt ( (1 + y[i]) * (101-y[i]) / (float) loops );
	
	/*
	for ( i = 1; i <= n; i++ ) 
		printf ("  %d %f %f %f\n", i, x[i], y[i], sig[i]);
	*/

//	covar = matrix(2,2);
//	alpha = matrix(2,2);
	lista[1] = 1;
	lista[2] = 2;
	dxsq = 0.;
	alamda = -1.;

	i = 0;
	CFitFunction fintgauss(ErrorType);
	while ( nstable < 1  &&  nltot < 10 )
	{
		nltot++;
		
		if (mrqmin (x,y,sig,nc,a,2,lista,2,covar,alpha,&xsq,&fintgauss,&alamda) == -1)
			return ( 7 );

		if ( i == 0 ) {
			fxsq = lxsq = xsq;
			i = 1;
		}
		if ( (lxsq - xsq) / xsq < 0.01 && xsq < fxsq - 0.0001) 
			nstable++;

		lxsq = xsq;
	}

	alamda = 0.;
	mrqmin (x, y, sig, nc, a, 2, lista, 2, covar, alpha, &xsq, &fintgauss, &alamda);
	pars[0] = a[1];
	pars[1] = a[2];
	errs[0] = sqrt(covar(0,0));
	errs[1] = sqrt(covar(1,1)); 
	*Rchisq = xsq / (nc - 2.0);

	if ( xsq / (nc-2.0) > 6.)  return ( 8 );  /* don't allow chisq to exceed 6 per fit point */

	free_vector (a, 1, 2);
//	free_matrix (covar, 2, 2);
//	free_matrix (alpha, 2, 2);
	free_vector (sig, 1, n);
	free_vector (y, 0, n);
	
	return ( 0 );

}


CCurveFitter::GammQ(float a, float x) {
	float gamser, gammcf, gln;

	if (x<0.0 || a <= 0.0)
		AfxMessageBox("Error in finding gamma function.", MB_ICONEXCLAMATION|MB_OK);
	if (x < (a+1.0)) {
		gser(&gamser, a, x, &gln);
		return 1.0-gamser;
	} else {
		gcf(&gammcf, a, x, &gln);
		return gammcf;
	}
}


float CCurveFitter::gammln(float xx)
{
   float x,tmp,ser;
   static float cof[6]={76.18009173,-86.50532033,24.01409822,
       -1.231739516,0.120858003e-2, -0.536382e-5};
   int j;
   x=xx-1.0;
   tmp=x+5.5;
   tmp -= (x+0.5)*log(tmp);
   ser=1.0;
   for (j=0;j<=5;j++) {
       x += 1.0;
       ser += cof[j]/x;
   }
   return -tmp+log(2.50662827465*ser);
}

void CCurveFitter::gser(float * gamser, float a, float x, float * gln)
/* Returns the incomplete gamma function P(a,x) evaluated by its series 
   representation as gamser.  */
{
   int m;
   float sum,del,ap;

   *gln=gammln(a);
   if (x<= 0.0) {
       *gamser=0.0;
       return;
   } else {
       ap=a;
       del=sum=1.0/a;
       for (m=1; m<=ITMAX; m++) {
           ap += 1.0;
           del *= x/ap;
           sum += del;
           if (fabs(del) < fabs(sum)*EPS) {
               *gamser=sum*exp(-x+a*log(x)-(*gln));
               return;
           }
       }
		AfxMessageBox("a too large, ITMAX too small in routine GSER.", MB_ICONEXCLAMATION|MB_OK);
       return;
   }
}

void CCurveFitter::gcf(float * gammcf,float a, float x, float * gln)
{
   int n;
   float gold=0.0,g,fac=1.0,b1=1.0;
   float b0=0.0,anf,ana,an,a1,a0=1.0;

   *gln=gammln(a);
   a1=x;
   for (n=1;n<=ITMAX;n++) {
       an=(float) n;
       ana=an-a;
       a0=(a1+a0*ana)*fac;
       b0=(b1+b0*ana)*fac;
       anf=an*fac;
       a1=x*a0+anf*a1;
       b1=x*b0+anf*b1;
       if (a1) {
           fac=1.0/a1;
           g=b1*fac;
           if (fabs((g-gold)/g) < EPS) {
               *gammcf=exp(-x+a*log(x)-(*gln))*g;
               return;
           }
        gold=g;
        }
   }
	AfxMessageBox("a too large, ITMAX too small in routine GCF.", MB_ICONEXCLAMATION|MB_OK);
	return;
}

/* ====================================================== */
/*     mrqmin                                             */
/* ====================================================== */													  

int CCurveFitter::mrqmin (float  *x,
			float  *y,
			float  *sig,
			int     ndata,
			float  *a,
			int     ma,
			int     *lista,
			int     mfit,
			Tmatrix<float> covar,
			Tmatrix<float> alpha,
			float  *chisq,
			CFitFunction * functionType,
			float  *alamda )
							  
{
	
	int k, kk, j, ihit;
	static float *da, *atry,*beta,ochisq;
	Tmatrix<float> oneda;
	if(*alamda < 0.0) {
		oneda.SetSize(mfit,1);
		atry=vector(1,ma);
		da=vector(1,ma);
		beta=vector(1,ma);
		kk=mfit+1;
		for(j=1;j<=ma;j++) {
			ihit=0;
			for(k=1;k<=mfit;k++)
				if(lista[k]==j) ihit++;
			if(ihit==0)
				lista[kk++] = j;
			else if (ihit>1) {
				AfxMessageBox("Bad LISTA permutation in MRQNMIN-1", MB_ICONEXCLAMATION|MB_OK);
				return -1;
			}
		}
		if(kk!=ma+1) {
			AfxMessageBox("Bad LISTA permutation in MRQNMIN-2", MB_ICONEXCLAMATION|MB_OK);
			return -1;
		}
		*alamda = 0.001;
		mrqcof(x,y,sig,ndata,a,ma,lista,mfit,alpha,beta,chisq,functionType);
		ochisq=(*chisq);
	}
	for(j=1;j<=mfit;j++) {
		for(k=1;k<=mfit;k++) covar(j,k) = alpha(j,k);
		covar(j,j)=alpha(j,j)*(1.0+(*alamda));
		oneda(j,1) = beta[j];
	}
	if(gaussj(covar,mfit,oneda,1)==-1) return -1;
	for(j=1;j<=mfit;j++)
		da[j]=oneda(j,1);
	if (*alamda == 0.0) {
		covsrt(covar,ma,lista,mfit);
		free_vector(beta,1,ma);
		free_vector(da,1,ma);
		free_vector(atry,1,ma);
//		free_matrix(oneda,mfit,1);
		return 0;
	}
	for(j=1;j<=ma;j++) atry[j]=a[j];
	for(j=1;j<=mfit;j++)
		atry[lista[j]] = a[lista[j]]+da[j];
	mrqcof(x,y,sig,ndata,atry,ma,lista,mfit,covar,da,chisq,funcs);
	if(*chisq < ochisq) {
		*alamda *= 0.1;
		ochisq = (*chisq);
		for (j=1;j<=mfit;j++) {
			for(k=1;k<=mfit;k++) alpha(j,k) = covar(j,k);
			beta[j] = da[j];
			a[lista[j]]=atry[lista[j]];
		}
	} else {
		*alamda *= 10.0;
		*chisq = ochisq;
	}
	return 0;
}


void CCurveFitter::mrqcof(float x[],float y[], float sig[], int ndata, float a[],
								  int ma, int lista[], int mfit, Tmatrix<float> alpha, float beta[],
								  float * chisq,  CFitFunction * functionType)  //funcs)
/* used by mrqmin to evaluate the linearized fitting matrix alpha[1..mfit][1..mfit] and
   vector beta[1..mfit] as in (14.4.8). */
{
   	int k,j,i;
   	float ymod,wt,sig2i,dy,*dyda;

   	dyda=vector(1,ma);
   	for(j=1;j<=mfit;j++) {
   		for(k=1;k<=j;k++) alpha(j,k) = 0.0;
   		beta[j]=0.0;
	}
   	*chisq=0.0;
   	for(i=1;i<=ndata;i++) {
   		fintgauss(x[i],a,&ymod,dyda,ma);
   		sig2i=1.0/(sig[i]*sig[i]);
   		dy=y[i]-ymod;
   		for (j=1;j<=mfit;j++) {
   			wt = dyda[lista[j]]*sig2i;
   			for(k=1;k<=j;k++)
   				alpha[j][k] += wt*dyda[lista[k]];
   			beta[j] += dy*wt;
   		}
   		(*chisq) += dy*dy*sig2i;
   	}
   	for(j=2;j<=mfit;j++)
   		for(k=1;k<=j-1;k++) alpha(k,j) = alpha(j,k);
   	free_vector(dyda,1,ma);
}


float * CCurveFitter::vector(int min, int max) {
	float * vec;
	vec = new float [max-min+1];
	return vec;
}

void CCurveFitter::free_vector(float * vec, int min, int max) {
	delete [] vec;
}

void CCurveFitter::fintgauss (float x, float a[], float *y, float dyda[], int na)

{

  double  x1 = ( x - a[1] ) / a[2];
//  double  erfcc ();
  double  sqrt2 = 1.414213562;

  *y = 50. * erfcc (x1 / sqrt2);

  if ( na > 0 ) 
  {
	  dyda[1] = 100.0 * exp ( -x1*x1 / 2.0 ) / ( 2.0 * sqrt2 * a[2] );
  	  dyda[2] = dyda[1] * x1;
  }

}


int CCurveFitter::gaussj(Tmatrix<float> a, int n, Tmatrix<float> b, int m)
/* linear equation solution by Gauss-Jordan elimination, p. 36 Num Rec. */
{
	int *indxc, *indxr, *ipiv;
	int i,icol,irow,j,k,l,ll;
	double big,dum,pivinv;

	indxc=ivector(1,n);
	indxr=ivector(1,n);
	ipiv=ivector(1,n);
	for (j=1;j<=n;j++) ipiv[j]=0;
	for (i=1;i<=n;i++) {
		big = 0.0;
		for(j=1;j<=n;j++)
			if (ipiv[j] != 1)
				for (k=1;k<=n;k++) {
					if (ipiv[k] == 0) {
						if (fabs(a(j,k)) >= big) {
							big=fabs(a(j,k));
							irow=j;
							icol=k;
						}
					} else {
						if (ipiv[k] > 1) {
							nrerror("GAUSSJ: Signular Matrix-1");
							return -1;
						}
					}
				}
		++(ipiv[icol]);
		if (irow != icol) {
			for(l=1;l<=n;l++) SWAP(a(irow,l),a(icol,l))
			for(l=1;l<=m;l++) SWAP(a(irow,l),a(icol,l))
		}
		indxr[i] = irow;
		indxc[i]=icol;
		if (a[icol][icol] == 0.0) {
			nrerror("GAUSSJ: Singular Matrix-2");
			return -1;
		}
		pivinv=1.0/a[icol][icol];
		a[icol][icol]=1.0;
		for (l=1;l<=n;l++) a(icol,l) *= pivinv;
		for (l=1;l<=m;l++) b(icol,l) *= pivinv;
		for (ll=1;ll<=n;ll++)
			if (ll != icol) {
				dum=a(ll,icol);
				a(ll,icol)=0.0;
				for (l=1;l<=n;l++) a(ll,l) -= a(icol,l)*dum;
				for (l=1;l<=m;l++) b(ll,l) -= b(icol,l)*dum;
			}
	}
	for (l=n;l>=1;l--) {
		if (indxr[l] != indxc[l])
			for (k=1;k<=n;k++)
				SWAP(a(k,indxr[l]),a(k,indxc[l]));
	}
	free_ivector(ipiv,1,n);
	free_ivector(indxr,1,n);
	free_ivector(indxc,1,n);
	return 0;
}