//////////////////////////CData class implementation ///////////////////////
#include "stdafx.h"
#include <afxtempl.h>

#include "Data.h"
#include "Aldas.h"
#include "sys_pub.h"
#include "hostspec.h"
#include "Sys_priv.h"
#include "Message.h"
#include "DSPcard.h"
#include "LLcard.h"
//#include "Hybrid.h"
#include "module.h"

#include "hal.h"

extern CALDASApp theApp;

CData::CData(int moduleIndex) {
	HAL * hal = theApp.GetHal();
	dsp = hal->GetDsp(moduleIndex);

	Module * module = hal->GetModule(moduleIndex);
	
	link = module->link;

	triggers = 0;

	DspDacVoltage[0] = hal->GetVatt(moduleIndex);
	DspDacVoltage[1] = hal->GetVref(moduleIndex);
	DspDacVoltage[2] = 0;		// this DAC is unused at present.
	DspDacVoltage[3] = hal->GetCalibration(moduleIndex);

	LLCardDacVoltage[0] = hal->GetThreshold(moduleIndex);
	LLCardDacVoltage[1] = hal->GetThreshold(moduleIndex);
	LLCardDacVoltage[2] = hal->GetThreshold(moduleIndex);
	LLCardDacVoltage[3] = hal->GetThreshold(moduleIndex);

	strobeDelay = hal->GetStrobeDelay(moduleIndex);
	numChannels = hal->GetNumChannels(moduleIndex);
	counterValue = 0;		// this information is irrelevant.
	eventLength = 0;
	biasVoltage=hal->GetModule(moduleIndex)->GetBiasVoltage();
	alignmentDelay=hal->GetAlignmentDelay(moduleIndex);

	hits = new unsigned int [numChannels];
	multipleHits = new unsigned int [numChannels];

	for (int i=0; i<numChannels; i++) {
		hits[i] = 0;
		multipleHits[i] = 0;
	}
}


CData::CData(DSP* d, int l, int cb) {
	LLCard * llCard = NULL;
	Module * module = NULL;
	hits = NULL;
	multipleHits = NULL;

	int i;
	dsp = d;
	link=l;
	triggers=0;
	biasVoltage=-1.0;
	alignmentDelay=128;

	if (d) {
		for (i=0; i<4; i++) {
			DspDacVoltage[i] = d->GetDACVoltage(i);
		}
		llCard=d->GetLLCard();
		cbFileName = llCard->GetControlBlock(cb);
		if (llCard) {
			for (i=0; i<4; i++) {
				LLCardDacVoltage[i] = llCard->GetLLDACVoltage(i);
			}
			module = llCard->GetModule(link/2);
			strobeDelay = llCard->GetStrobeDelay();
			if (module) {
				numChannels=module->GetNumChannels();
				counterValue = module->GetCounterValue();
				eventLength = module->GetEventLength();
				hits=new unsigned int [numChannels];
				multipleHits=new unsigned int [numChannels];
				for (int j=0; j<numChannels; j++) {
					hits[j] = 0;
				   multipleHits[j] = 0;
				}
			} else {
				hits=NULL;
				multipleHits = NULL;
				numChannels=0;
			}
		}
	}
//	TRACE ("CData created.\n");
}
	
CData::~CData() {
	if (hits)	delete [] hits;
	if (multipleHits) delete [] multipleHits;

//	TRACE ("CData destroyed.\n");
}


void CData::Serialize(CArchive& ar)
{ // stores and reads hits, link, triggers, numChannels only, not the DSP
  CString buf;
  char line[100];
  int i, j;
  float version=1.1;

  if (ar.IsStoring()) {// storing code here:
    	if (!hits) {
			TRACE ("CData::Serialize: No hit array!\n");
			return;
		}
      buf.Format ("# ALDAS %5.1f\n", VERSION);
      ar.WriteString(buf);
		for (i=0; i<4; i++) {
			buf.Format("%f %f\n", DspDacVoltage[i], LLCardDacVoltage[i]);
			ar.WriteString(buf);
		}
		buf.Format("%i %i\n", strobeDelay, counterValue);
		ar.WriteString(buf);

		buf.Format("$%s\n", cbFileName);
		ar.WriteString(buf);
		buf.Format ("%i %i %i\n", link, triggers, numChannels);  // FIXME: this line
		ar.WriteString (buf);									// isn't getting written to the file.
		TRACE("%s", buf);
		for (i=0; i<numChannels/8; i++) {
			j=i*8;
			buf.Format("%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\n", 
		   hits[j  ], hits[j+1], hits[j+2], hits[j+3], 
		   hits[j+4], hits[j+5], hits[j+6], hits[j+7]);
		   ar.WriteString (buf);
		}
		// new for version 1.3:
		buf.Format("%i %f alignmentDelay, biasVoltage\n", alignmentDelay, biasVoltage);
		ar.WriteString(buf);
		for (i=0; i<numChannels/8; i++) {
			j=i*8;
			buf.Format("%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\n", 
			multipleHits[j  ], multipleHits[j+1], multipleHits[j+2], multipleHits[j+3], 
			multipleHits[j+4], multipleHits[j+5], multipleHits[j+6], multipleHits[j+7]);
			ar.WriteString (buf);
		}
	}	else	{		//loading code here
		ar.ReadString(line, 100);
		if (line[0]=='#') { // version 
			sscanf (line, "# ALDAS %f", &version);
			ar.ReadString(line, 100);
		}
		for (i=0; i<4; i++) {
			sscanf(line, "%f %f", &(DspDacVoltage[i]), &(LLCardDacVoltage[i]));
			ar.ReadString (line, 100);
		}
		sscanf(line, "%i %i", &strobeDelay, &counterValue);

		ar.ReadString (line, 100);
		if (version>1.1) {
			cbFileName = & (line[1]); // skip $ sign
		} else {
			cbFileName = & (line[0]);
			if (line[0]==0) cbFileName.Empty();
		}
	   ar.ReadString (line, 100);
	   sscanf(line, "%i %u %i", &link, &triggers, &numChannels);

		// Read the hits data from the file.
	   if (hits) delete [] hits;
	   hits=new unsigned int [numChannels];
	   for (i=0; i<numChannels/8; i++) {
	      j=i*8;
	      ar.ReadString (line, 100);
	      sscanf(line, "%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u",
	        &(hits[j  ]), &(hits[j+1]), &(hits[j+2]), &(hits[j+3]), 
	        &(hits[j+4]), &(hits[j+5]), &(hits[j+6]), &(hits[j+7]));
	   }

		// Now read the multipleHits data from file.
		if (multipleHits) delete [] multipleHits;
		multipleHits=new unsigned int [numChannels];
		biasVoltage=-1.0;
		alignmentDelay=128;
		if (version > 1.201) {
		   ar.ReadString (line, 100);
		   sscanf(line, "%i %f", &alignmentDelay, &biasVoltage);
			for (i=0; i<numChannels/8; i++) {
				j=i*8;
				ar.ReadString (line, 100);
				sscanf(line, "%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u",
				  &(multipleHits[j  ]), &(multipleHits[j+1]), &(multipleHits[j+2]), &(multipleHits[j+3]), 
				  &(multipleHits[j+4]), &(multipleHits[j+5]), &(multipleHits[j+6]), &(multipleHits[j+7]));
			}
		}
	}
}

void CData::SetTriggers (unsigned int t)
{
	triggers=t;
}

unsigned int CData::GetTriggers()
{
	return triggers;
}

int CData::GetLink()
{ // returns the dsp link (0, 1, 2, 3).
	return link;
}

int CData::GetNumChannels()
{
	return numChannels;
}

double CData::GetLLDacVoltage(int i) {
	return LLCardDacVoltage[i];
}

int CData::GetEventLength()
{
	return eventLength;
}

int CData::GetCounterValue()
{
	return counterValue;
}

int CData::GetStrobeDelay()
{
	return strobeDelay;
}

float CData::GetDSPDacVoltage(int i)
{
	return DspDacVoltage[i];
}

CString CData::GetCBName()
{
	return cbFileName;
}

void CData::SetAlignmentDelay(int d)
{
  alignmentDelay = d;
}

void CData::SetBiasVoltage(float b)
{
  biasVoltage = b;
}

float CData::GetBiasVoltage()
{
  return biasVoltage;
}

int CData::GetAlignmentDelay()
{
	return alignmentDelay;
}
