// TakeDataDlg.cpp : implementation file

// User of this dialog box must set the value of fifo.
// If fifo is between 0 and 3, the calibrate message is
// used to collect data.
// Otherwise, the burst mode is used.

// If the value of fifo is not between 0 and 3, then we want to do
// a start of burst, end of burst cycle.  This means we are taking
// data from external triggers.  This is useful when we have a source.

#include "stdafx.h"
#include <afxtempl.h>
#include <time.h>

#include "ALDAS.h"
#include "PlaceDoc.h"
#include "ALDASDoc.h"
#include "Data.h"
#include "Message.h"
#include "DSPcard.h"
#include "LLcard.h"
#include "module.h"

#include "hal.h"

#include "TakeDataDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CTakeDataDlg dialog
extern CALDASApp theApp;


CTakeDataDlg::CTakeDataDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTakeDataDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTakeDataDlg)
	//}}AFX_DATA_INIT
	drawn=0;		// counts for TDStart
	triggers=0; // number of triggers obtained
   istop=0;		// don't stop
 //  hitType = 0;
	fifo=0;

	hal = theApp.GetHal();
} 


void CTakeDataDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTakeDataDlg)
	DDX_Control(pDX, IDC_RATE, rateBox);
	DDX_Control(pDX, IDC_PROGRESS, progress);
	DDX_Control(pDX, IDC_TRIGGERS, trigBox);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTakeDataDlg, CDialog)
	//{{AFX_MSG_MAP(CTakeDataDlg)
	ON_BN_CLICKED(IDC_RESET, OnReset)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_TD_START, OnTDStart)
END_MESSAGE_MAP()

// CTakeDataDlg message handlers


long CTakeDataDlg::OnTDStart(UINT wParam, long lParam)
{
	MSG winMess;
	clock_t startTime;
	clock_t duration = 2 * CLOCKS_PER_SEC;  // wait for 2 seconds

	CString buf;

	beginTime = clock();
	hal->StartTakingData(data, moduleIndex, maxTriggers, triggerSource);

	do {
		startTime = clock();
		while ((data->dsp->GetFlag(3)==0)&&(istop==0)) {
			if (::PeekMessage(&winMess, NULL, 0, 0, PM_REMOVE)) {
				::TranslateMessage(&winMess);
				::DispatchMessage (&winMess);
			}
			if (clock() > startTime + duration) {
				if (AfxMessageBox("TIME OUT while waiting for the 'data ready' flag.", 
					MB_RETRYCANCEL | MB_ICONSTOP) == IDRETRY) {
					startTime = clock();
				}  else { // cancel
					istop=-1;
					CDialog::OnCancel(); // delete dialog
				}
			}
		}
		//TRACE("Data ready %i\n", data->dsp->GetFlag(3));
		
		try {
			hal->TakeData();
		} catch(int errorNumber) {
			istop = -1;
		}
		if (istop != 0) break; // EXIT this do loop

		triggers = data->GetTriggers();
//		TRACE("triggers = %i\n", triggers);

		rate = float(clock() - beginTime );
		if (rate>0.0) rate = triggers / rate * CLOCKS_PER_SEC;
		progress.SetPos(100*triggers/maxTriggers);
		if (rate<10.0) {
			buf.Format("%7.2f", rate);
		} else if (rate<100.0) {
			buf.Format("%7.1f", rate);
		} else {
			buf.Format("%7.0f", rate);
		}
		rateBox.SetWindowText(buf);
		buf.Format("%i", triggers);
		trigBox.SetWindowText(buf);
		histo.Invalidate();
		histo.SendMessage(WM_PAINT);

		// allow windows to process messages:
		for (int i=0; i<3; i++){// loop to allow more than 1 message to be processed:
			if (::PeekMessage(&winMess, NULL, 0, 0, PM_REMOVE)){
				::TranslateMessage(&winMess);
				::DispatchMessage (&winMess);
			}
		}
	} while ((triggers<maxTriggers)&&(istop==0));
	
	hal->FinishTakingData();

	// got enough data
	if (istop<0) {
		CDialog::OnCancel(); // delete dialog
	} else {
	  CDialog::OnOK(); // delete dialog
	}
	return 0L;
}

void CTakeDataDlg::OnReset() 
{
	// reset hits[] to zero and continue to take data
	for (int i=0; i<data->GetNumChannels(); i++) data->hits[i]=0;
	triggers=0;
	data->SetTriggers(0);
	beginTime = clock();
	histo.Invalidate();
	histo.SendMessage(WM_PAINT);
}

void CTakeDataDlg::OnStop() 
{
	// got enough data
	istop=1; // this will stop the loop in OnTDStart
}


BOOL CTakeDataDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	histo.SubclassDlgItem (IDC_HISTO, this);
	histo.pData=data->hits;
	histo.length=maxChannels;
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CTakeDataDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	if (drawn==0) { // post message after window becomes visible:
		PostMessage(WM_TD_START);
		drawn++;
	}
	// Do not call CDialog::OnPaint() for painting messages
}

void CTakeDataDlg::OnCancel() 
{
	istop=-1;
}

void CTakeDataDlg::Init(CData* d, int m, int trigs, int source)
{
	// (re)initialize the variables of this dlg
	drawn=0; // counts for TDStart
	triggers=0; // number of triggers obtained
   istop=0; // don't stop

	maxTriggers = trigs;
	triggerSource = source;
	moduleIndex = m;
	data=d;
	if (!data) {
		TRACE("CTakeDataDlg::Init: data=NULL!\n");
		return;
	}
	maxChannels=data->GetNumChannels();
}
