// ALDASDoc.cpp : implementation of the CALDASDoc class

#include "stdafx.h"
#include <afxcoll.h>
#include <afxtempl.h>
#include "ALDAS.h"


#include "message.h"
#include "dspcard.h"
#include "llcard.h"

#include "hal.h"
#include "module.h"
#include "PlaceDoc.h"
#include "ALDASDoc.h"
#include "MessageDlg.h"
#include "NoiseScanDlg.h"
#include "SelectDspDlg.h"
#include "SendMessageDlg.h"
#include "ConfigDlg.h"


extern CALDASApp theApp;


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// NumDocsOpen is a static variable.  It is initialized once when the program runs.
UINT CALDASDoc::NumDocsOpen = 0;

IMPLEMENT_DYNCREATE(CALDASDoc, CDocument)

BEGIN_MESSAGE_MAP(CALDASDoc, CDocument)
	//{{AFX_MSG_MAP(CALDASDoc)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_SEND_MESSAGE, OnSendMessage)
	ON_COMMAND(ID_VIEW_ALL_MESSAGES, OnViewAllMessages)
	ON_COMMAND(ID_VIEW_ERROR_MESSAGES, OnViewErrorMessages)
	ON_COMMAND(ID_NEVER_SHOW_MESSAGES, OnNeverShowMessages)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ALL_MESSAGES, OnUpdateViewAllMessages)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ERROR_MESSAGES, OnUpdateViewErrorMessages)
	ON_UPDATE_COMMAND_UI(ID_NEVER_SHOW_MESSAGES, OnUpdateNeverShowMessages)
	ON_COMMAND(ID_MODIFY_CONFIG, OnModifyConfig)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CALDASDoc construction/destruction

CALDASDoc::CALDASDoc()
{
	hal = new HAL;
	// window placement:
	place.length=sizeof (WINDOWPLACEMENT);
	place.showCmd=SW_NORMAL; //nCmdShow;
	place.flags=WPF_SETMINPOSITION;
	place.ptMinPosition=CPoint(0,0);
	place.ptMaxPosition=CPoint(-::GetSystemMetrics (SM_CXBORDER), -::GetSystemMetrics (SM_CYBORDER)); 

/*	CDC myDC;
	CString buf = hal->systemPath;
	int width = myDC.GetTextExtent(buf).cx;
	if (width < 200) width = 200;
	int height = hal->dspArray.GetSize() * 45 + 50;*/
	place.rcNormalPosition=CRect(0, 0, 400, 200);
}

CALDASDoc::~CALDASDoc()
{
	delete hal;
}

// CALDASDoc serialization

void CALDASDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())	{	// storing code here
		hal->Serialize(ar);
	}	else	{					// loading code here:
		hal->Serialize(ar);
	}

}

// CALDASDoc diagnostics

#ifdef _DEBUG
void CALDASDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CALDASDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

// CALDASDoc commands

BOOL CALDASDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (NumDocsOpen >= 1) {
		AfxMessageBox("Only one configuration file can be open at a time.", MB_ICONSTOP|MB_OK);
		return FALSE;
	} else {
		if (!CDocument::OnOpenDocument(lpszPathName))
			return FALSE;
	}
	
	// TODO: Add your specialized creation code here
	NumDocsOpen++;
	return TRUE;
}

void CALDASDoc::OnCloseDocument() 
{
	// TODO: Add your specialized code here and/or call the base class
	if (NumDocsOpen > 0) NumDocsOpen--;
	CDocument::OnCloseDocument();
}


HAL * CALDASDoc::GetHal() {
	return hal;
}

int CALDASDoc::IsDspActive() {
	int dspActive = false;
//	CALDASDoc * pDoc=GetDocument();
	int numDsps = hal->dspArray.GetSize();
		for (int i=0; i<numDsps; i++) {
			DSP * nextDsp = hal->dspArray[i];
			if (dspActive = nextDsp->IsCardReady())
					break;
		}
	return dspActive;
}

void CALDASDoc::OnFileSave() 
{
	if (AfxMessageBox("Do you want to write the curent DSP state into the configuration file?",
		MB_ICONQUESTION|MB_OKCANCEL) == IDCANCEL) return;
	CDocument::OnFileSave();	
}

void CALDASDoc::OnSendMessage() 
{
	int result;
	UINT ciw, link, p1, p2, p3;

	CSelectDspDlg selectDlg;
	selectDlg.theFilter = CSelectDspDlg::None;
	if (selectDlg.DoModal() != IDOK)
		return;
	link = selectDlg.link;
	DSP * dsp = selectDlg.dsp;
	// TODO: Add your command handler code here
	SendMessageDlg messDlg;
	messDlg.ciw = 0;
	messDlg.param1 = 0;
	messDlg.param2 = 0;
	messDlg.param3 = 0;
	messDlg.if_verbose = TRUE;
	messDlg.always = FALSE;
	if (messDlg.DoModal() != IDOK)
		return;
	else {
		ciw = messDlg.ciw;
		p1 = messDlg.param1;
		p2 = messDlg.param2;
		p3 = messDlg.param3;
		try {
			result = dsp->ShortMessage(ciw, link, p1, p2, p3);
		} catch (int messResult) {
			if (messResult < 0) return;
		}
		int temp_mode = theApp.mDisplayMessages;
		theApp.mDisplayMessages = 0;
		
		if (messDlg.always || (messDlg.if_verbose && dsp->GetFlag(VERBOSE_FLAG) == 1))
			dsp->GetReply();
		theApp.mDisplayMessages = temp_mode;
	}
}

void CALDASDoc::OnViewAllMessages()
{
	theApp.mDisplayMessages = 0;
}

void CALDASDoc::OnViewErrorMessages() 
{
	theApp.mDisplayMessages = 1;
}

void CALDASDoc::OnNeverShowMessages() 
{
	theApp.mDisplayMessages = 2;
}

void CALDASDoc::OnUpdateViewAllMessages(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(theApp.mDisplayMessages == 0);
	pCmdUI->Enable(IsDspActive());
}

void CALDASDoc::OnUpdateViewErrorMessages(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(theApp.mDisplayMessages == 1);
	pCmdUI->Enable(IsDspActive());
}

void CALDASDoc::OnUpdateNeverShowMessages(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(theApp.mDisplayMessages == 2);
	pCmdUI->Enable(IsDspActive());
}

void CALDASDoc::OnUpdateSendMessage(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(IsDspActive());
}

void CALDASDoc::OnUpdateGetMessage(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(IsDspActive());
}

void CALDASDoc::OnModifyConfig()
{
	CConfigDlg dlg(NULL, hal);
	dlg.DoModal();
	UpdateAllViews(NULL);
	SetModifiedFlag(TRUE);
}