// ALDASView.cpp : implementation of the CALDASView class

#include "stdafx.h"
#include <afxtempl.h>
#include "ALDAS.h"


#include "message.h"
#include "dspcard.h"
#include "LLCard.h"
//#include "hybrid.h"

#include "hal.h"
#include "module.h"

#include "PlaceDoc.h"
#include "ALDASDoc.h"
#include "ALDASView.h"
#include "DSPDlg.h"
#include "ChangeDspDlg.h"
#include "SelectDspDlg.h"
#include "SendMessageDlg.h"
#include "ConfigDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CALDASView

IMPLEMENT_DYNCREATE(CALDASView, CView)

BEGIN_MESSAGE_MAP(CALDASView, CView)
	//{{AFX_MSG_MAP(CALDASView)
	ON_COMMAND(ID_SYSTEM_CODE, OnSystemCode)
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(ID_DSP_BUTTON, ID_DSP_BUTTON16, OnDSP)
	ON_COMMAND_RANGE(ID_INIT_BUTTON, ID_INIT_BUTTON16, OnInitButton)
END_MESSAGE_MAP()

// CALDASView construction/destruction

CALDASView::CALDASView() :	mTopofFirstRow(25)
{
	mRowHeight = 45;
	mColumnWidth = 80;

}

CALDASView::~CALDASView()
{
	for (int i=0; i<mNumDataRows; i++) {
		delete DSPButton[i];
		delete initButtons[i];
	}
}

void CALDASView::OnInitialUpdate() {
	CALDASDoc * pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	mNumDataRows = pDoc->hal->dspArray.GetSize();
	for (int i=0; i<mNumDataRows; i++) {
		DSPButton[i] = new CButton;
		initButtons[i] = new CBitmapButton;
	}
	AddDSPButtons();
}

void CALDASView::AddDSPButtons() {
	CALDASDoc * pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	DSP * thisCard;
	CRect rect1, rect2;
	CBitmap dspNormal, dspGrey;

	for (int i=0; i<mNumDataRows; i++) {
		rect1.SetRect(5, (i+1)*mRowHeight + 5, 70, (i+1)*mRowHeight + 25);
//		rect2.SetRect(95, (i+1)*mRowHeight, 124, (i+1)*mRowHeight + 30);
		rect2.SetRect(95, (i+1)*mRowHeight+5, 95+20, (i+1)*mRowHeight + 5+20);
		thisCard = pDoc->hal->dspArray[i];
		if (initButtons[i]->m_hWnd == NULL)
			initButtons[i]->Create("A",BS_OWNERDRAW | BS_PUSHBUTTON | WS_VISIBLE | WS_CHILD, 
				rect2, this, ID_INIT_BUTTON + i);
		if (thisCard->IsCardReady()) {
			if (DSPButton[i]->m_hWnd == NULL)
				DSPButton[i]->Create(thisCard->GetName(),BS_PUSHBUTTON | WS_VISIBLE | 
											WS_CHILD, rect1, this, ID_DSP_BUTTON + i);
			initButtons[i]->LoadBitmaps(IDB_CHECK_MARK_UP, IDB_CHECK_MARK_DOWN);
			initButtons[i]->Invalidate();
		} else {
			if (DSPButton[i]->m_hWnd == NULL)
				DSPButton[i]->Create(thisCard->GetName(),BS_PUSHBUTTON | WS_VISIBLE | 
											WS_CHILD | WS_DISABLED, rect1, this, ID_DSP_BUTTON + i);
			initButtons[i]->LoadBitmaps(IDB_DISABLED_CHECK_UP, IDB_DISABLED_CHECK_DOWN);
			initButtons[i]->Invalidate();
			TRACE ("AddDSPButtons: WS_DISABLED\n");
		}
	}
}

// CALDASView drawing

void CALDASView::OnDraw(CDC* pDC)
{
	int i, j, dipSetting;
	CBitmap dipUp, dipDown;
	CDC memoryDC;
	DSP * dsp;
	CString buf;

	CALDASDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CRect myWinRect, rect;
	GetClientRect(&myWinRect);

	// First draw the separators and the column titles.
	pDC->MoveTo(0, 25);
	pDC->LineTo(myWinRect.right, 25);
	pDC->TextOut(5, 5, "Cards");
	pDC->MoveTo(mColumnWidth, 0);
	pDC->LineTo(mColumnWidth, myWinRect.bottom-25);
	pDC->TextOut(mColumnWidth + 5, 5, "Initialized");
	pDC->MoveTo(mColumnWidth*2, 0);
	pDC->LineTo(mColumnWidth*2, myWinRect.bottom-25);
	pDC->TextOut(mColumnWidth*2 + 5, 5, "DIP Switch");
	pDC->MoveTo(0, myWinRect.bottom-25);
	pDC->LineTo(myWinRect.right, myWinRect.bottom-25);

	// draw the dip switches
	dipUp.LoadBitmap(ID_DIP_SWITCH_UP);
	dipDown.LoadBitmap(ID_DIP_SWITCH_DOWN);
	memoryDC.CreateCompatibleDC(pDC);
	mNumDataRows = pDoc->hal->dspArray.GetSize();
	for (i=0; i<mNumDataRows; i++) {
		dsp=pDoc->hal->dspArray[i];
		dipSetting=dsp->GetBaseAddress() / 0x80000;
		for (j=4; j>=0; j--){
			if ((dipSetting >> j) & 1) {
				memoryDC.SelectObject(dipUp);
			} else {
				memoryDC.SelectObject(dipDown);
			}
			pDC->BitBlt(mColumnWidth*2 + 5 + (4-j)*15, (i+1)*mRowHeight + 5, 
							15, 19, &memoryDC, 0, 0, SRCCOPY);
		}
	}

	// Finally, write the name of the system file being used.
	buf.Format ("DSP system code file: '%s'.", pDoc->hal->systemPath);
	pDC->TextOut (myWinRect.left+6, myWinRect.bottom-20, buf);
}

// CALDASView printing

BOOL CALDASView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CALDASView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CALDASView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


#ifdef _DEBUG
void CALDASView::AssertValid() const
{
	CView::AssertValid();
}

void CALDASView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CALDASDoc* CALDASView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CALDASDoc)));
	return (CALDASDoc*)m_pDocument;
}
#endif //_DEBUG

afx_msg void CALDASView::OnDSP( UINT nID )
{ // bring up the dialog for dsps

	CALDASDoc * pDoc=GetDocument();
	int dspNumber=nID-ID_DSP_BUTTON;
	DSP * dsp = pDoc->hal->dspArray[dspNumber];

	CDSPDlg dlg(this, dsp);

	if (dlg.DoModal() != IDOK)
		return;
}

afx_msg void CALDASView::OnInitButton(UINT nID) {
	CALDASDoc * pDoc=GetDocument();
	int dspNumber=nID-ID_INIT_BUTTON;
	pDoc->hal->dspArray[dspNumber]->InitializeCard(pDoc->hal->systemPath);
	pDoc->UpdateAllViews(NULL);
}

void CALDASView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	// TODO: Add your specialized code here and/or call the base class
	CALDASDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int numDsps = pDoc->hal->dspArray.GetSize();
	if (mNumDataRows < numDsps) {
		for (int i=mNumDataRows; i<numDsps; i++) {
			DSPButton[i] = new CButton;
			initButtons[i] = new CBitmapButton;
		}
		mNumDataRows = numDsps;
		AddDSPButtons();
	} else if (mNumDataRows > numDsps) {
		// then delete the extra buttons...
		for (int i=numDsps; i<mNumDataRows; i++) {
			delete DSPButton[i];
			delete initButtons[i];
		}
		mNumDataRows = numDsps;
	}

	// And make sure the button states match the current state of the DSPs.
	DSP * dsp;
	for (int i=0; i<mNumDataRows; i++) {
		dsp = pDoc->hal->dspArray[i];
		if (!dsp->IsCardReady()) {
			DSPButton[i]->EnableWindow(FALSE);
		} else
			DSPButton[i]->EnableWindow(TRUE);
	}
	Invalidate(TRUE);
}

// change the system code file and reinitialize the dsp cards:
void CALDASView::OnSystemCode() 
{
	CALDASDoc * pDoc=GetDocument();
	static char BASED_CODE szFilter[] = "DSP System Files (*.asc)|*.asc|All Files (*.*)|*.*||";
	CFileDialog dlg(TRUE, ".asc", pDoc->hal->systemPath, OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST, szFilter);
	if (dlg.DoModal() != IDOK) return;

	pDoc->hal->systemPath = dlg.GetFileName(); //better: dlg.GetPathName();
	// Now boot the DSP cards:
	for (int dsp=0; dsp < pDoc->hal->dspArray.GetSize(); dsp++) {
		DSP* myCard = (DSP *) pDoc->hal->dspArray[dsp];
		myCard->InitializeCard(LPCTSTR(pDoc->hal->systemPath));
	}
	pDoc->SetModifiedFlag(TRUE);
	Invalidate();
	pDoc->UpdateAllViews(NULL);
}