// calibrationScanDoc.cpp : implementation of the CCalibrationScanDoc class
//

#include "stdafx.h"
#include <afxtempl.h>
#include <math.h>
#include "ALDAS.h"
#include "MainFrm.h"

#include "PlaceDoc.h"
#include "ScanDoc.h"
#include "Axis.h"
#include "CalibrationScanDoc.h"
#include "SelectModuleDlg.h"
//#include "DSPDlg.h"
//#include "ThresholdDlg.h"
#include "TakeDataDlg.h"
#include "CalibrationDlg.h"
#include "Data.h"
#include "Message.h"
#include "DSPcard.h"
#include "llcard.h"
#include "module.h"
#include "hal.h"

//#include "CurveFitter.h"
#include "fitLSQ.h"

extern CALDASApp theApp;


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CCollectDataDoc

IMPLEMENT_DYNCREATE(CCalibrationScanDoc, CDocument)

BEGIN_MESSAGE_MAP(CCalibrationScanDoc, CDocument)
	//{{AFX_MSG_MAP(CCalibrationScanDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CCollectDataDoc construction/destruction

CCalibrationScanDoc::CCalibrationScanDoc()
{
	numParam = 2;
	axis.SetSize(2);
	par.SetSize(2);
	err.SetSize(2);
	chi2.SetSize(1); // will be changed when thrPoints is known
	thr=NULL;
	mean=NULL;
	meanErr=NULL;
	response=NULL;
	CRect client;
	AfxGetMainWnd()->GetClientRect (client);
	place.rcNormalPosition=CRect(-1, -1, client.Width()/22*10, client.Height()-50);
}

CCalibrationScanDoc::~CCalibrationScanDoc()
{
	delete [] mean;
	delete [] meanErr;
	delete [] thr;
	delete response; // Axis
}

BOOL CCalibrationScanDoc::OnNewDocument()
{
	int i, k;
	int triggers;
//	CString cbOld; // name of old control block # 3
	CString buf;
	CMainFrame* pFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus= &pFrame->m_wndStatusBar;

	if (!CDocument::OnNewDocument())
		return FALSE;

	///////////////// Select the dsp and link. //////////////////////
	CSelectModuleDlg dlg;
	if (dlg.DoModal() != IDOK) return FALSE;
	int moduleIndex = dlg.moduleNumber;

	hal->ModifySettings(moduleIndex);

	///////////////// Get min, max, and thrInc. //////////////////
	CCalibrationDlg calDlg;
	calDlg.calMin = 0.0;
	calDlg.calMax = 1.5;
	calDlg.calInc = 0.25;
	calDlg.thrMin = 0.05;
	calDlg.thrMax = 0.3;
	calDlg.thrInc = 0.05;
	calDlg.triggers = 200;
	calDlg.line0 = FALSE;
	calDlg.line1 = TRUE;
	calDlg.line2 = FALSE;
	calDlg.line3 = TRUE;
	if (calDlg.DoModal() != IDOK)	return FALSE;

	calLine[0] = calDlg.line0;
	calLine[1] = calDlg.line1;
	calLine[2] = calDlg.line2;
	calLine[3] = calDlg.line3;
	thrInc = calDlg.thrInc;
	thrMax = calDlg.thrMax;
	thrMin = calDlg.thrMin;
	calInc = calDlg.calInc;
	calMax = calDlg.calMax;
	calMin = calDlg.calMin;

	triggers = calDlg.triggers;
	thrPoints = (UINT) ((thrMax - thrMin)/thrInc + 1.5);
	calPoints = (UINT) ((calMax - calMin)/calInc + 1.5);
	numPoints = calPoints * thrPoints;
	dataArray.SetSize(numPoints);

	numParam = 2*thrPoints;
	axis.SetSize(2*thrPoints);
	par.SetSize(2*thrPoints);
	err.SetSize(2*thrPoints);
	numChi2=thrPoints; // numChi2 is stored in ScanDoc, thrPoints isn't
	chi2.SetSize(numChi2);

	numChannels=hal->GetNumChannels(moduleIndex);
	if (numChannels<1) return FALSE;
	module = hal->GetModule(moduleIndex);
	moduleName = module->GetModuleName();
	badChannel=new BOOL [numChannels];
	for (i=0; i<numChannels; i++) badChannel[i]=module->badChannel[i];

	float oldThreshold = hal->GetThreshold(moduleIndex);
	float oldCalibration = hal->GetCalibration(moduleIndex);

	///////////////// Take the data /////////////////////////////////
	CTakeDataDlg take;
	take.maxTriggers = triggers;

	float curCalibration, curThreshold = thrMin;

	for (int j=0; j<thrPoints; j++) {
		par[2*j+0] = new float [numChannels];
		par[2*j+1] = new float [numChannels];
		err[2*j+0] = new float [numChannels];
		err[2*j+1] = new float [numChannels];
		chi2[j]    = new float [numChannels];
		hal->SetThreshold(moduleIndex, curThreshold);
		curCalibration = calMin;
		for (int i=0; i<calPoints; i++) {
			hal->SetCalibration(moduleIndex, curCalibration);
			dataArray[i+calPoints*j] = new CData (moduleIndex);
			for (k=0; k<numChannels; k++)	dataArray[i+calPoints*j]->hits[k] = 0;
			buf.Format("Calibration scan %i of %i: Threshold = %f V, Calibration = %f V\n", 
				j*calPoints+i, numPoints, curThreshold, curCalibration);
			pStatus->SetPaneText (0, buf);
			take.Init(dataArray[i+calPoints*j], moduleIndex, triggers, HAL::INTERNAL /* internal? */);
			if (take.DoModal()!=IDOK)  {
				pStatus->SetPaneText (0, "");
				hal->SetCalibration(moduleIndex, oldCalibration);
				hal->SetThreshold(moduleIndex, oldThreshold);
				return FALSE; // canceled
			}
			curCalibration += calInc;
		}
		curThreshold += thrInc;
	}

	pStatus->SetPaneText (0, "");

	hal->SetCalibration(moduleIndex, oldCalibration);
	hal->SetThreshold(moduleIndex, oldThreshold);

	AnalyzeData();

	heading.Format("Calibration Scan of '%s', %d Triggers", 
						module->GetModuleName(), dataArray[0]->GetTriggers());
	SetModifiedFlag(TRUE);
	return TRUE;
}


void CCalibrationScanDoc::AnalyzeData() {
	int i, j;
	float curCalibration, curThreshold;
	float *tempY, *tempX, *sig;
	UINT hits = 0;
	float ave_eff, a[4], m1;
	int trigs = 0, j1;
	int nonzero; // count # of points with >1 hits
	int lista[3] = {0, 1, 2};
	float lamda, oldChi2;
	int nfit, iThr, mrq;
	void intgausscomp (float, float*, float*, float*, int);
	float **alpha, **covar;

	
	tempY = new float [calPoints];
	tempX = new float [calPoints];
	sig = new float [calPoints];
	thr = new float [thrPoints];
	mean = new float [thrPoints];
	meanErr = new float [thrPoints];
	
	covar=matrix(1, 2, 1, 2);
	alpha=matrix(1, 2, 1, 2);

	curThreshold = thrMin;
	for (iThr=0; iThr<thrPoints; iThr++) {
		thr[iThr] = curThreshold;
		for (i=0; i<numChannels; i++) {
			par[2*iThr+0][i] = 0;
			par[2*iThr+1][i] = 0;
			if (module->badChannel[i]) {
				chi2[iThr][i] = -2;
			} else {
				nonzero = 0;
				curCalibration = calMin;
				for (j=0; j<calPoints; j++) {
					trigs = dataArray[j+iThr*calPoints]->GetTriggers();
					hits = dataArray[j+iThr*calPoints]->hits[i];
					ave_eff = float (hits)/float(trigs);
					if (ave_eff > 0.5) nonzero++;
					tempY[j] = ave_eff;
					sig[j] = sqrt(ave_eff*(1.0-ave_eff)/trigs)/ave_eff; // Binomial stat.
					if (sig[j]<0.01)	sig[j]=0.01; 
					tempX[j] = curCalibration;
					//TRACE ("x=%f, y=%f\n", tempX[j], tempY[j]);
					curCalibration += calInc;
				}
				//TRACE ("\n");
				j1=0;
				while ((j1<calPoints)&&(tempY[j1]<0.5)) j1++;
				if ((nonzero>1)&&(calLine[i%4])&&(j1<calPoints)) { // more than 1 non-zero efficiency					
					if (j1==0){
						a[1]=tempX[j1];
						m1=20;
					} else { // linear interpolation
						m1=(tempY[j1]-tempY[j1-1])/(tempX[j1]-tempX[j1-1]);
						a[1]=tempX[j1-1]+(0.5-tempY[j1-1])/m1;
						if (a[1]<-30) a[1]=-30;
						if (a[1]>300) a[1]=300;
					}
					a[2]=1.0/sqrt(2.0*3.141592)/fabs(m1);
					if (a[2]<0.0) a[2] = 0.0;
					if (a[2]>4.0) a[2] = 4.0;
					//TRACE ("a1=%f, a2=%f, j1=%i\n", a[1], a[2], j1);

					lamda = -1.0;
					nfit=0;
					chi2[iThr][i]=1E10;
					do {
						oldChi2=chi2[iThr][i];
						mrq=mrqmin(tempX-1, tempY-1, sig-1, calPoints, a, 2, lista, 2, covar,
								 alpha, &(chi2[iThr][i]), &intgausscomp, &lamda);
						if (mrq==-1) break; // fit doesnt work
	//					TRACE ("mrqmin: nfit = %i, lamda=%f, a1=%f, a2=%f, chi2 = %f\n", 
	//						nfit, lamda, a[1], a[2], chi2[i]);
						nfit++;
					} while ((nfit<2)||((nfit<10)&&(oldChi2-chi2[iThr][i]>0.1)));
					// calculate cov matrix:
					if (mrq!=-1) {
						lamda = 0.0;
						mrqmin(tempX-1, tempY-1, sig-1, calPoints, a, 2, lista, 2, covar,
								 alpha, &(chi2[0][i]), &intgausscomp, &lamda);
					} else {
						chi2[iThr][i]=-3;
					}

					par[2*iThr+0][i] = a[1];
					par[2*iThr+1][i] = a[2];
					err[2*iThr+0][i] = sqrt(covar[1][1]);
					err[2*iThr+1][i] = sqrt(covar[2][2]);
				} else { // not ((nonzero>1)&&(calLine[i%4]))
					par[2*iThr+0][i] = 0.5*(calMin+calMax);
					par[2*iThr+1][i] = (calMax-calMin)/calPoints;
					err[2*iThr+0][i] = 0;
					err[2*iThr+1][i] = 0;
					chi2[iThr][i] = -1;
				}
			}
		} 
		axis[2*iThr+0]=new CAxis ("Response", numChannels, par[2*iThr+0], err[2*iThr+0], chi2[iThr]);
		axis[2*iThr+1]=new CAxis ("Sigma", numChannels, par[2*iThr+1], err[2*iThr+1], chi2[iThr]);

		mean[iThr] = axis[2*iThr+0]->Mean(0, numChannels, meanErr[iThr]);
		// convert to charge:
		float high=Volt2Charge (mean[iThr]+meanErr[iThr]);
		float low =Volt2Charge (mean[iThr]-meanErr[iThr]);
		mean[iThr] = Volt2Charge (mean[iThr]);
		meanErr[iThr] = 0.5*(high-low);
		TRACE ("Thr = %f, Mean = %f, Sigma = %f\n",curThreshold, mean[iThr], meanErr[iThr]);
		curThreshold += thrInc;
	}

	// fit parabola to response curve:
	a[1]=0.0;
	a[2]=(mean[thrPoints-1]-mean[0]) / (thr[thrPoints-1]-thr[0]);
	a[3]=0.0;
	float chi2Parabola;
	int listb[4] = {0, 1, 2, 3};
	free_matrix (covar, 1, 2, 1, 2);
	free_matrix (alpha, 1, 2, 1, 2);
	covar=matrix(1, 3, 1, 3);
	alpha=matrix(1, 3, 1, 3);

	lamda = -1.0;
	nfit=0;
	chi2Parabola=1E10;
	do {
		oldChi2=chi2Parabola;
		mrq = mrqmin(thr-1, mean-1, meanErr-1, thrPoints, a, 3, listb, 3, 
				covar, alpha, &chi2Parabola, &parabola, &lamda);
		if (mrq) {
			TRACE ("ERROR in mrqmin!\n");
		}
		TRACE ("mrqmin: nfit = %i, lamda=%f, a1=%f, a2=%f, a3=%f, chi2 = %f\n", 
			nfit, lamda, a[1], a[2], a[3], chi2Parabola);
		nfit++;
	} while ((nfit<2)||((nfit<10)&&(oldChi2-chi2Parabola>0.1)));

	for (i=0; i<3; i++) coeff[i]=a[i+1];

	// one response diagram:
	response= new CAxis ("Mean response [fC]", thrPoints, mean, meanErr, NULL);
	threshold = CAxis ("Threshold [V]", thrPoints, thr, NULL, NULL);

	free_matrix (covar, 1, 3, 1, 3);
	free_matrix (alpha, 1, 3, 1, 3);

	delete [] tempY;
	delete [] tempX;
	delete [] sig;

}




// CCalibrationScanDoc serialization

void CCalibrationScanDoc::Serialize(CArchive& ar)
{
  CString buf;
  int i;
  float version;
	if (ar.IsStoring())	{
		CScanDoc::Serialize(ar);
		buf.Format ("# ALDAS %5.1f\n", VERSION);
		ar.WriteString(buf);
		buf.Format("%i thrPoints\n", thrPoints);
		ar.WriteString (buf);
		for (i=0; i<thrPoints; i++){
			buf.Format("%f \t %f \t %f\n", thr[i], mean[i], meanErr[i]);
			ar.WriteString (buf);
		}
		buf.Format("%f %f %f parabola\n", coeff[0], coeff[1], coeff[2]);
		ar.WriteString (buf);

	} else {
		CScanDoc::Serialize(ar);
		ar.ReadString(buf);
		if (buf[0]=='#') { // version 
			sscanf (buf, "# ALDAS %f", &version);
			ar.ReadString(buf);
		}
		sscanf (buf, "%i", &thrPoints);
		if (thr) TRACE ("CCalibrationScanDoc Serialize: thr[] exists already.\n");
		thr = new float [thrPoints];
		mean = new float [thrPoints];
		meanErr = new float [thrPoints];
		for (i=0; i<thrPoints; i++){
			ar.ReadString (buf);
			sscanf (buf, "%f %f %f", &(thr[i]), &(mean[i]), &(meanErr[i]));
		}
		if (response) TRACE ("CCalibrationScanDoc Serialize: response exists already.\n");
		response= new CAxis ("Mean response [fC]", thrPoints, mean, meanErr, NULL);
		threshold = CAxis ("Threshold [V]", thrPoints, thr, NULL, NULL);
		ar.ReadString (buf);
		sscanf(buf, "%f %f %f", &(coeff[0]), &(coeff[1]), &(coeff[2]));
	}
}


// CCalibrationScanDoc diagnostics

#ifdef _DEBUG
void CCalibrationScanDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CCalibrationScanDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

// CCalibrationScanDoc commands

float CCalibrationScanDoc::Volt2Charge(float volt)
{
  /* converts calibration voltage in volt to charge in femto Coulomb */
  float uncor; /* uncorrected charge */
  
  uncor = volt/25.0*84.0; /* femto Coulomb */
  return uncor/(1.0+exp(-0.299*(uncor+5.302)));
}
