
#include "stdafx.h"
#include <afxtempl.h>

#define VXIWIN32
#include "nivxi.h"

#include "ALDAS.h"

#include "llcard.h"

#include "hal.h"
#include "module.h"
#include "message.h"
#include "dspcard.h"
#include "TccCard.h"

#include "data.h"

#include "lbic.h"

#include "PlaceDoc.h"
#include "ALDASDoc.h"
#include "LLDialog.h"
#include "DSPDlg.h"
#include "TccDlg.h"

extern CALDASApp theApp;

HAL::HAL() {
	systemInitialized = FALSE;
	triggerSource = INTERNAL;
	dataModule = 0;

//	dspArray.SetSize(16);		// 16 slots in a VME crate
	llcardArray.SetSize(16);
//	moduleArray.SetSize(64);	// 4 modules/DSP
	data = NULL;
}

HAL::~HAL() {
	int numCards = dspArray.GetSize();
	DSP * myCard;

	for (int i=0; i<numCards; i++) {
		myCard = (DSP *) dspArray[i];
		delete myCard;
	}
	dspArray.RemoveAll();
	CloseVXIlibrary();
//	theApp.interfaceReady = FALSE;
}


// These functions change DAC values in the DSP or LL Card.
// Um, the DAC corresponding to link 0 is DAC #2.
// The DAC corresponding to link 2 is DAC #0.
// And likewise, DAC 3 and DAC #1 are flipped.
void HAL::SetThreshold(int module, float thresh) {
	LLCard * theCard = moduleArray[module]->llCard;
	int link = moduleArray[module]->link;
	if (theCard) {
		// This fixes the DAC to link problem.
		link = (link + 2) % 4;
		try {
		theCard->SetLLDACVoltage(link, thresh);
		} catch (int errCode) {
			ReportError(errCode);
		}
	}
//	MainDocIsDirty();
}

float HAL::GetThreshold(int module) {
	LLCard * theCard = moduleArray[module]->llCard;
	int link = moduleArray[module]->link;
	link = (link + 2) % 4;

	float threshold = 0;
	try {
		threshold = theCard->GetLLDACVoltage(link);
	} catch (int errCode) {
		ReportError(errCode);
	}
	return threshold;
}

void HAL::SetVatt(int module, float vatt) {
	DSP * theDsp = ModuleToDsp(module);
	try {
		theDsp->SetDACVoltage(0, vatt);
	} catch (int errCode) {
		ReportError(errCode);
	}
//	MainDocIsDirty();
}

float HAL::GetVatt(int module) {
	DSP * theDsp = ModuleToDsp(module);
	try {
		return theDsp->GetDACVoltage(0);
	} catch (int errCode) {
		ReportError(errCode);
	}
}

void HAL::SetVref(int module, float vref) {
	DSP * theDsp = ModuleToDsp(module);
	try {
		theDsp->SetDACVoltage(1, vref);
	} catch (int errCode) {
		ReportError(errCode);
	}
//	MainDocIsDirty();
}

float HAL::GetVref(int module) {
	DSP * theDsp = ModuleToDsp(module);
	try {
		return theDsp->GetDACVoltage(1);
	} catch (int errCode) {
		ReportError(errCode);
	}
}

void HAL::SetCalibration(int module, float cal) {
	DSP * theDsp = ModuleToDsp(module);
	try {
		theDsp->SetDACVoltage(3, cal);
	} catch (int errCode) {
		ReportError(errCode);
	}
//	MainDocIsDirty();
}

float HAL::GetCalibration(int module) {
	DSP * theDsp = ModuleToDsp(module);
	try {
		return theDsp->GetDACVoltage(3);
	} catch (int errCode) {
		ReportError(errCode);
	}
}

// This function is for any strips module.
int HAL::GetNumChannels(int index) {
	return moduleArray[index]->GetNumChannels();
}

// This function is specific to the LBIC-CDP chips.
// It should have a more generic name, like GetPipelineDelay
int HAL::GetStrobeDelay(int index) {
	LLCard * theCard = moduleArray[index]->llCard;
	try {
		return theCard->GetStrobeDelay();
	} catch (int errCode) {
		ReportError(errCode);
	}
}

void HAL::SetStrobeDelay(int index, int delay) {
	LLCard * theCard = moduleArray[index]->llCard;
	try {
		theCard->SetStrobeDelay(delay);
	} catch (int errCode) {
		ReportError(errCode);
	}
//	MainDocIsDirty();
}

// These functions are very useful.
int HAL::GetAlignmentDelay(int index) {
	TccCard * tcc = moduleArray[index]->llCard->ParentDSP->GetTccCard();
	try {
		return tcc->GetDelay();
	} catch (int errCode) {
		ReportError(errCode);
	}
}

void HAL::SetAlignmentDelay(int index, int delay) {
	TccCard * tcc = moduleArray[index]->llCard->ParentDSP->GetTccCard();
	try {
		tcc->SetDelay(delay);
	} catch (int errCode) {
		ReportError(errCode);
	}
//	MainDocIsDirty();
}

// Call this function when you want the user to be able
// to make changes to the hardware settings.
void HAL::ModifySettings(int index) {
	DSP * theDsp = ModuleToDsp(index);

	CDSPDlg dspDlg(NULL, theDsp);
	dspDlg.DoModal();
//	MainDocIsDirty();
}

// These three functions make it easy to collect data from
// any sort of module. Call StartTakingData one time only,
// then keep calling TakeData until you have enough triggers.
// Finally, call FinishTakingData one time when you are finished.

void HAL::StartTakingData(CData * d, int module, int trigs, int source) {
	data = d;
	dataModule = module;
	numTriggers = trigs;
	triggerSource = source;

	DSP * dsp = ModuleToDsp(module);
	LLCard * llcard = moduleArray[module]->llCard;


	// If the user wants to use internal triggers, load the CalAll control
	// block, into position 0.
	if (triggerSource == INTERNAL) {
		LLCard * theLL = moduleArray[dataModule]->llCard;
		oldControlBlock = theLL->GetControlBlock(0);
		if (oldControlBlock != theApp.cbCalAll)
			theLL->LoadControlBlock(0, theApp.cbCalAll);
		int link = moduleArray[dataModule]->link;
		dsp->DoCalibrate(link, 0, 10);
	}

	// If external, then enter 'burst' mode.
	if (triggerSource == EXTERNAL) {
		int flagSuccess;
		flagSuccess = dsp->SetFlag(EOB_SIG_FLAG, 0);	// Clear the EOB flag.
		flagSuccess = dsp->SetFlag(SOB_SIG_FLAG, 1);	// Set the SOB flag.
	}
}

int HAL::TakeData() {
	unsigned int * mess;
	DSP * dsp = ModuleToDsp(dataModule);

	try {
		int link = moduleArray[dataModule]->link;
		if (triggerSource == INTERNAL) {
			dsp->DoCalibrate(link, 0, 20);
		}
		if (triggerSource == EXTERNAL) {
			int flagSuccess;
			flagSuccess = dsp->SetFlag(SOB_SIG_FLAG, 0);		// Clear the SOB flag.
			flagSuccess = dsp->SetFlag(EOB_SIG_FLAG, 1);		// Set the EOB flag.
		}
		mess  =  dsp->DataTransfer(link, 2); // link 0, data format 2
		if (triggerSource == EXTERNAL) {
			int flagSuccess;
			if (flagSuccess = dsp->SetFlag(EOB_SIG_FLAG, 0));	// Clear the EOB flag.
			if (flagSuccess = dsp->SetFlag(SOB_SIG_FLAG, 1));	// Set the SOB flag.
		}
	} catch(int errorNumber) {
		ReportError(errorNumber);
		return errorNumber;
	}

	DecodeData(mess);
	return 0;
}

void HAL::FinishTakingData() {
	DSP * dsp = ModuleToDsp(dataModule);

	if (triggerSource == INTERNAL) {
		LLCard * theLL = moduleArray[dataModule]->llCard;
		if (oldControlBlock != theApp.cbCalAll)
			theLL->LoadControlBlock(0, oldControlBlock); // restore original cb
	}
	if (triggerSource == EXTERNAL) {
		int flagSuccess;
		flagSuccess = dsp->SetFlag(SOB_SIG_FLAG, 0);		// Clear the SOB flag.
		flagSuccess = dsp->SetFlag(EOB_SIG_FLAG, 1);		// Set the EOB flag.
	}

	// Clear the data pointer, since it may no longer be valid.
	data = NULL;
	dataModule = 0;
	triggerSource = INTERNAL;
}

void HAL::DecodeData(UINT * inData) {
	moduleArray[dataModule]->DecodeData(data, inData);
}

void HAL::InitializeSystem() {
	// Initialize the VXI library first.
	DSP * myCard;
	if (systemInitialized == FALSE) {
		int status = InitVXIlibrary();
		if (status == -1) {
			TRACE ("InitVXIlibrary status=%i\n", status);
			AfxMessageBox("Cannot initialize VXI library.", MB_ICONEXCLAMATION|MB_OK);
		}
		if (status == 1) {
			AfxMessageBox("VXI library already initialized.", MB_ICONINFORMATION|MB_OK);
		}
	}
	systemInitialized = TRUE;

	// Then boot the DSP cards.
	for (int dsp=0; dsp < dspArray.GetSize(); dsp++) {
		myCard = (DSP *) dspArray[dsp];
		myCard->InitializeCard(LPCTSTR(systemPath));
	}

	// Finally, do any initialization that the modules require.
	int size = moduleArray.GetSize();
	for (int i=0; i<moduleArray.GetSize(); i++) {
		if (GetDsp(i)->IsCardReady())
			moduleArray[i]->InitializeModule();
	}
}

void HAL::InitializeArrays() {
	LLCard * llcard;
	Module * module;
	int j=0;
	for (int i=0; i<dspArray.GetSize(); i++) {
		llcard = dspArray[i]->GetLLCard();
		if (llcard) {
			llcardArray.Add(llcard);
			for (j=0; j<4; j++) {
				module = llcard->GetModule(j);
				if (module)
					moduleArray.InsertAt(0, module);
			}
		}
	}
//	int size = moduleArray.GetSize();
}

int HAL::IsModuleActive() {
	int moduleActive = FALSE;
	for (int i=0; i<dspArray.GetSize(); i++) {
		LLCard * llcard = dspArray[i]->GetLLCard();
		if (llcard && dspArray[i]->IsCardReady()) {
			Module * m1 = llcard->GetModule(0);
			Module * m2 = llcard->GetModule(1);
			Module * m3 = llcard->GetModule(2);
			Module * m4 = llcard->GetModule(3);
			if (m1 || m2 || m3 || m4) {
				moduleActive = TRUE;
				break;
			}
		}
	}
	return moduleActive;
}

// Temporary functions, giving backwards compatibility...
DSP * HAL::GetDsp(int module) {
	return moduleArray[module]->llCard->ParentDSP;
}

Module * HAL::GetModule(int moduleIndex) {
	return moduleArray[moduleIndex];
}

// Private functions.
DSP * HAL::ModuleToDsp(int module) {
	return moduleArray[module]->llCard->ParentDSP;
}

/*void HAL::MainDocIsDirty() {
	CDocTemplate* templ=theApp.GetDocTemplate("ALDAS");
	POSITION docPos=templ->GetFirstDocPosition();
//	if (!docPos) return FALSE;

	CALDASDoc* mainDoc=(CALDASDoc*)templ->GetNextDoc(docPos);
	mainDoc->SetModifiedFlag(TRUE);
}*/

void HAL::ReportError(int error) {
	TRACE("Caught an exception in HAL with error code %i\n", error);
}

void HAL::Serialize(CArchive& ar)
{
	CFileFind systemFile;
	DSP * myCard;
	BOOL fileFound;
	char buf[500];
	int i=0;
	char line[80];
	float version=1.1;


	if (ar.IsStoring())	{

		sprintf (line, "# ALDAS %5.1f\n", VERSION);
		ar.WriteString(line);

		sprintf (line, "$%s", systemPath);
		ar.WriteString (line);

		sprintf (line, "\n%i Number of DSPs\n", dspArray.GetSize());
		ar.WriteString (line);

		for (int dsp=0; dsp < dspArray.GetSize(); dsp++) {
			myCard = (DSP *) dspArray[dsp];
			myCard->Serialize(ar);
		}

	}	else	{

		ar.ReadString (line, 80);
		if (line[0]=='#') { // version 
			sscanf (line, "# ALDAS %f", &version);
			ar.ReadString(line, 80);
		}

		systemPath=&(line[1]); // skip the $
		fileFound = systemFile.FindFile(systemPath);
		if (fileFound == 0) {
			CFile * configFile = ar.GetFile();
			CString configFileName = configFile->GetFileName();
			i = sprintf(buf, "I'm sorry, I cannot open a new main document at this time\n");
			i += sprintf(buf+i, "because I could not find the file %s.  Please check\n", systemPath);
			i += sprintf(buf+i, "that the file name in the configuration file %s\n", LPCSTR(configFileName));
			i += sprintf(buf+i, "matches the name of the system file.");
			AfxMessageBox(buf, MB_ICONEXCLAMATION|MB_OK);
			return;
		}

		ar.ReadString (line, 80);
		int numdspCards;
		sscanf (line, "%i", &numdspCards);

		int oldNumCards = dspArray.GetSize();
		if (oldNumCards > 0) {
			for (int i=0; i<oldNumCards; i++) {
				myCard = (DSP *) dspArray[i];
				delete myCard;
			}
			dspArray.RemoveAll();
		}

		for (UINT dsp=0; dsp<numdspCards; dsp++) {
			myCard = new DSP(dsp);
			myCard->Serialize(ar);
			dspArray.Add(myCard);
		}
		InitializeArrays();
		InitializeSystem();
	}
}