#include "stdafx.h"

#include "aldas.h"
#include "message.h"
#include "messageDlg.h"

extern CALDASApp theApp;

typedef int int32;

CMessage::CMessage() {
//	TRACE ("CMessage::CMessage() was called.\n");
	message_base = new UINT[MESS_BLOCKSIZE];
	clock_tick = 0;
	port_image=0;
//	mDisplayMessages = 1;
}


CMessage::~CMessage() {
	delete [] message_base;
}

//void CMessage::DisableAllMessages(BOOL onoff) {
//	mMessagesAllowed = !onoff;
//}

int CMessage::ShortMessage(UINT ciw, UINT dsp_num, UINT link, 
	UINT a1, UINT a2, UINT a3)
{
	int result;
// writes a short message into the message base and send it to the DSP.
	int len; // message length, depends on message type ciw.

// Define the length of the message in this switch statement.
	switch (ciw) {
		case INITIALIZE:
		case GET_LL_STATUS:
		case GET_DSP_STATUS:
		case RESET_FIFOS:
		case GET_TCC_STATUS:
			len = HEADLEN;
			break;

		case SET_STROBE_DELAY:
		case TRANSFER_DATA:
		case SELECT_TRIGGER_MODE:
		case SET_COUNTER_VAL:
		case CHANGE_CLOCK:
		case SELECT_ALIGN_DELAY:
		case GET_EXTERNAL_TRIGGERS:
		case GET_SOFT_TRIGGERS:
			len = HEADLEN+1;
			break;
	
		case SET_DAC:
		case SET_LL_DAC:
		case EXECUTE_CONTROL_BLOCK:
			len = HEADLEN+2;
			break;
	
		case ECHO:
		case CALIBRATE:
		case MESSAGE_TEST:
			len = HEADLEN+3;
			break;


		default:
			//len = HEADLEN;
			TRACE ("ShortMessage: Unknown CIW=%i\n", ciw);
			AfxMessageBox("Unknown CIW, message not sent.", MB_ICONEXCLAMATION|MB_OK);
			return -1;
//			ASSERT (FALSE);
	}

	// message header:
   message_base[0] = ciw;					// ciw: command identifier word
   message_base[1] = len;					// len: message length
   message_base[2] = dsp_num;				// dsp: dsp #
   message_base[3] = 1;						// dsp_dipsw: dsp dip switch
   message_base[4] = link;					// channel: unit/channel #
   message_base[5]= 0;						// serviced by dsp dsp
   message_base[6] = 0;						// spare: spare[5]
   message_base[7] = 0;						// spare: spare[5]
   message_base[8] = 0;						// spare: spare[5]
	message_base[9] = 0;				      // spare: spare[5]
	message_base[10] = 0;				      // spare: spare[5]
	// parameters:
   message_base[11] = a1;				   // 1st parameter 
   message_base[12] = a2;					// 2nd parameter 
   message_base[13] = a3;					// 3rd parameter 


	CMessageDlg dlg;
	dlg.base=(unsigned*)message_base;
	dlg.message_sender = this;
	if (theApp.mDisplayMessages == 0)
		dlg.DoModal();

	result = send_mess(message_base);
	if (result != 0)
		throw result;
	
	return result;
}


int CMessage::LongMessage(UINT ciw, UINT dsp_num, UINT link, UINT* ptr, UINT arraysize,
								  UINT nParams, UINT a1, UINT a2, UINT a3)
{
	int result;
	int len;

	
	switch (ciw) {
	case LOAD_CONTROL_BLOCK:
		len= HEADLEN + 2049;
		break;
	default:
		len =HEADLEN + 2049;
		TRACE ("LongMessage: Unknown CIW=%i\n", ciw);
		break;
	}


	message_base[0] = ciw;					// ciw: command identifier word
   message_base[1] = len;					// len: message length         
   message_base[2] = dsp_num;				// dsp: dsp #                  
   message_base[3] = 1;						// dsp_dipsw: dsp dip switch   
   message_base[4] = link;					// channel: unit/channel #     
   message_base[5]= 0;						// serviced by dsp dsp			
   message_base[6] = 0;						// spare: spare[5]            
   message_base[7] = 0;						// spare: spare[5]             
   message_base[8] = 0;						// spare: spare[5]             
	message_base[9] = 0;				      // spare: spare[5]             
	message_base[10] = 0;				   // spare: spare[5]

	// get extra parameters
	message_base[11] = a1;
	message_base[12] = a2;
	message_base[13] = a3;

	// now get the array.
	for (UINT i=0; i<arraysize; i++) {
		message_base[HEADLEN + nParams + i] = ptr[i];
	}
	
	// Display the message dlg if the menu is set appropriately.
	CMessageDlg dlg;
	dlg.base=(unsigned*)message_base;
	dlg.message_sender = this;
	if (theApp.mDisplayMessages == 0)
		dlg.DoModal();

		result = send_mess(message_base);
//	else result = -1;
	return result;
}


int CMessage::GetReply()
{
	CMessageDlg dlg;
	int status = 3;
	m_wait();
	if((status = get_mess()) != SUCCESS) {
		TRACE("Error in CMessage::GetReply,  perhaps message already received?\n");
		throw status;
	} else {
		dlg.base = (UINT*) message_base;
		// Check the display mode to see if we want to show this message.
		if (theApp.mDisplayMessages == 0)
			dlg.DoModal();
		else if (theApp.mDisplayMessages == 1)
			if ( message_base[0] == FATAL_ERROR 
					|| message_base[0] == ERROR_REPORT)
				dlg.DoModal();

		// this is for debugging purposes only. 
		/*
		if(message_base[0]==RESPONSE)
			TRACE("CMessage::GetReply Response Code = %ld\n",message_base[11]);
		else
	  		TRACE("CMessage::GetReply CIW is: %ld\n",message_base[0]);
		*/
	}
   return status;
}


void CMessage::dsp_pause(int time)
{
	int i;
	for (i=0; i<time; i++) {}
}



UINT CMessage::peek()
{
  if(io_base) {
    if(io_base->signals & (1 << MEXPORT)) {
      return (io_base->packet.message[0]);
    }
  }
  return 0;
}



/***********************************************************************/

int CMessage::sys_set(int signal)		/* sets a bit in signal word */
{	
//  extern dpr *io_base[];
  int x = 1;
  
  if ((signal < 0) || (signal > NUM_FLAGS))
    return(ENOSIG);
  io_base->signals |= (x << signal); 
  return(SUCCESS);
}

/***********************************************************************/

int CMessage::sys_clear(int signal)
{
//  extern dpr *io_base[];
  int x = 1;
  
  if((signal < 0) || (signal > NUM_FLAGS))
    return(ENOSIG);
  io_base->signals &= ~(x << signal); 
  return(SUCCESS);
}

/***********************************************************************/

int CMessage::sys_wait(int signal)  /*  wait a while, or timeout  */
{
  unsigned long time;
  int val;
  
  for(time = (long) simp_clock(); (long) simp_clock() - time < MESS_TIMEOUT; ) {
    if((val = look(signal)) == EABORT)
      return EABORT;
    if(val == ECONFLICT)
      continue;
    if(val)
      return SUCCESS;
  }
  TRACE ("sys_wait: time= %ld  %ld\n", time, (long) simp_clock()); 
  return ETIMEOUT;
}

/***********************************************************************/

int CMessage::sys_nwait(int signal)
{
  unsigned long time;
  int val;
  
  for(time = (long) simp_clock(); (long) simp_clock() - time < MESS_TIMEOUT; ) {
    if((val = look(signal)) == EABORT)
      return EABORT;
    if(val == ECONFLICT)
      continue;
    if(!val)
      return SUCCESS;
  }
  return ETIMEOUT;
}

int CMessage::look(int sig) {
	int hackvar;
	if (io_base->signals == (hackvar = io_base->signals))
		return ((hackvar >> sig) & 1);
	else
		return ((io_base->signals >> sig) & 1);
}

/***********************************************************************/

int CMessage::get_sigs( )
{
 // extern dpr *io_base[];
  return io_base->signals;
}

/**********************************************************************/

int CMessage::send_mess(UINT *pmess )
{
  
  int remain;
  int status;
  UINT *from, *to;
  int dest, time;
  static int mess_id = 0;
//  static int num_sent = 0;
//  static int num_err = 0;
  static int iam = 0;
 // extern dpr *io_base[];
  m_generic *message;
  
  message = (m_generic *) pmess;
  dest = MDEST(message);

/*  printf("send_mess: dest %x  io_base[dest] %x\n", dest, io_base[dest]); */

  remain = MSIZE(message);
  
  if(look(MEXPORT))
    return ECONFLICT;
  
  sys_set(MIMPORT);
  
  if(look(MEXPORT))
    return ECONFLICT;
  
  mess_id++;
  time = 0;
  
  DO_T_HEAD( );
  
  
  from = (UINT *) message;
  to = io_base->packet.message;
  
  while( remain > MTU ) {
    memcpy(to, from, 4*MTU);
    sys_set(DAV);
    remain -= MTU;
    from += MTU;
    if((status = sys_wait(ACK)) != SUCCESS)
      return status;
    sys_clear(DAV);
    if((status = sys_nwait(ACK)) != SUCCESS)
      return status;
  }
  
  memcpy(to, from, 4*remain);
  to += remain;
  zero(to, (MTU - remain));
  
  sys_set(EOM);
  sys_set(DAV);
  if((status = sys_wait(ACK)) != SUCCESS)
	return status;
  sys_clear(EOM);
  sys_clear(MIMPORT);
  sys_clear(DAV);
  if((status = sys_nwait(ACK)) != SUCCESS)
	return status;
  
  return(SUCCESS);
  
}

int CMessage::get_mess()
{
  
  UINT *to, *from;
  int remain, status;
//  extern int *message_base;
//  int timestamp, mess_id;
  /*      int maxtsize = 0x10000; */  /*  FIXME  */
  int maxtsize = 400000;
  
  from = io_base->packet.message;
  to = message_base;
  
  if(!look(MEXPORT ))
    return ENOMESS;
  
  if((status = sys_wait( DAV )) != SUCCESS )
    return status;
  remain = TSIZE();
  if(abs(remain) > maxtsize)
    return ETOOBIG;
  
//  timestamp = TTIME( dest );
//  mess_id = TMESSID( dest );
  
  while(remain > MTU) {
    memcpy(to, from, 4*MTU);
    sys_set(ACK );
    to += MTU; remain -= MTU;
    if((status = sys_nwait( DAV )) != SUCCESS)
      return status;
    sys_clear( ACK );
    if((status = sys_wait( DAV )) != SUCCESS)
      return status;
  }
  
    memcpy(to, from, 4*MTU);
  sys_set(ACK );
  if((status = sys_nwait( DAV )) != SUCCESS)
    return status;
  sys_clear(  ACK );
  
  return SUCCESS;
}


void CMessage::copy(UINT *from, UINT *to, int n )
{
  int c;
  
  if((!from) || (!to))
    return;
  
  if(n == 0)	
    return;
  
  for( c = n; c != 0; c--) 
    *to++ = *from++;
  
}

void CMessage::zero(UINT *to, int n)
{
  int c;
  
  if((!to) || !(n > 0))
    return;
  
  for( c = n; c != 0; c--)
    *to++ = 0;
  
}


int CMessage::com_abort()
{
  sys_set(ABORT);
  sys_wait(ACK);
  sys_nwait(ACK);
  sys_clear(ABORT);
  return(SUCCESS);
}

void CMessage::com_aborted(int dir )
{				
  
  sys_clear(EOM);
  if(dir == MEXPORT) {
    sys_clear(MEXPORT);
    sys_clear(DAV);
  }
  else {
    sys_clear(MIMPORT);
  }	                      
  sys_set( ACK);		/* Not too sure about this */
  sys_clear( ACK);
  
}

int CMessage::message_status(void)
{
  
  int stat = 0;
  int n;
  
  for(n = NUM_CHAN - 1; n >= 0; n--) {
    if( check_mess() == 0) {
      stat <<= 1;
      continue;
    }
    stat <<= 1;
    stat += 1;
  }
  return stat;
}


int CMessage::check_mess()
{
//  int val;
  if(io_base) {
    if(io_base->signals & (1 << MEXPORT))
      return 1;
  }
  return 0;
//  return val;
}

int CMessage::m_wait()
{
  int err;
  
  err = sys_wait(MEXPORT);
  if(err == SUCCESS)
    return SUCCESS;
  if(err == ETIMEOUT)
    return ETIMEOUT;
  
  return ENOMESS;
}

UINT CMessage::m_size()
{
  if(!io_base)
    return 0;
  
  return(io_base->packet.message[1]);
}




void CMessage::dsp_led( int on )  /* turns red LED on DSP's VME card on or off */
{
  int i;
  
  i = get_port();	/* get theoretical current value in port */
  
  i &= ~0x80;		/* clear bit 7 = turn LED on */
  if ( ! on )  i |= 0x80;	/* set   bit 7 = turn LED off */
  
  set_port( i );	/* write new value to port */
}



void CMessage::interrupt_dsp()    /* interrupts DSP */
{
  int i;
  
  i = get_port( );	/* get current value in port */
  
  i |= 0x2;			/* set   bit 1 */
  set_port(i);
  i &= ~0x2;		/* clear bit 1 */
  set_port(i);
  i |= 0x2;		/* set   bit 1 */
  set_port(i);
}


int CMessage::get_port( )      /* returns image of port */
{
  return( port_image ); /* return image of port */
}


void CMessage::set_port( int value )  /* writes value to port */
{
  
  * ( (int *) io_base + PORT_OFFSET ) = value; /* write to hardware */
  port_image = value; /* write to image */
  
}

long CMessage::simp_clock(void)
{
   clock_tick++;
//   printf("\nclock_tick = %d", clock_tick);
	return(clock_tick);
}