#include "stdafx.h"
#include <afxtempl.h>

#include "dsp.h"
#include "message.h"
#include "llcard.h"
#include "dspcard.h"

#include "aldas.h"
#include "module.h"

#include "hal.h"

extern CALDASApp theApp;

Module::Module() {
	link = 0;
	numChannels = 0;
	hitPolarity = 0;
	detectorAttached = FALSE;
	CString name = "";
	llCard = NULL;
	biasCard = NULL;
	badChannel = NULL;

//	HAL * hal = theApp.GetHal();
//	hal->moduleArray.InsertAt(0, this);

	TRACE("Module created\n");
}

Module::~Module() {
	delete badChannel;

/*	HAL * hal = theApp.GetHal();
	for (int i=0; i<hal->moduleArray.GetSize(); i++)
		if (hal->moduleArray[i] == this) break;
	hal->moduleArray.RemoveAt(i);*/

	TRACE("Module destroyed\n");
}

void Module::SetHitPolarity(int polarity) {
	hitPolarity = polarity;
}

int Module::GetHitPolarity() {
	return hitPolarity;
}

int Module::GetNumChannels() {
	return numChannels;
}

void Module::SetNumChannels(int number) {
	int tmp;
	numChannels = number;
	if (badChannel) delete badChannel;
	badChannel = new BOOL [number];
	for (int i=0; i< numChannels; i++) badChannel[i]=FALSE;
	// calculate counterValue and eventLength:
	tmp=25+numChannels+9*(numChannels/128-1);
	if (tmp%8 ==0) {
		counterValue=tmp/8;
	} else {
		counterValue=tmp/8+1;
	} 
	if (tmp%32 ==0) {
		eventLength=tmp/32;
	} else {
		eventLength=tmp/32+1;
	} 
}

int Module::GetEventLength () {
	return eventLength;
}

int Module::GetCounterValue() {
	return counterValue;
}

BOOL Module::IsDetectorAttached() {
	if (name == NULL)
		return TRUE;
	else
		return FALSE;
}

void Module::SetModuleName(CString& inName) {
	name = inName;
}

CString& Module::GetModuleName() {
	return name;
}

void Module::Serialize (CArchive& ar){
	char line[80];
	int i, j, numBad;
	float version=1.1;

	if (ar.IsStoring())	{		// storing code here
      sprintf (line, "# ALDAS %5.1f\n", VERSION);
      ar.WriteString(line);
		sprintf (line, "$%s\n", name);
		ar.WriteString (line);
		sprintf (line, "%i numChannels\n", numChannels);
		ar.WriteString (line);
		sprintf (line, "%i link\n", link);
		ar.WriteString (line);

		// count # of bad channels:
		numBad=0;
		for (i=0; i< numChannels; i++) 
			if (badChannel[i]) numBad++;
		sprintf (line, "%i bad channels\n", numBad);
		ar.WriteString (line);

		for (i=0; i< numChannels; i++) {
			if (badChannel[i]) {
				sprintf (line, "%i is bad\n", i);
				ar.WriteString (line);
			}
		}
		sprintf(line, "%i hit polarity\n", hitPolarity);
		ar.WriteString (line);
		sprintf(line, "%f bias Voltage\n", biasVoltage);
		ar.WriteString (line);
	} else {		//  loading code here
		ar.ReadString (line, 80);
		if (line[0]=='#') { // version 
			sscanf (line, "# ALDAS %f", &version);
//			ar.ReadString(line, 80);
		}
		ar.ReadString (line, 80);
		name = &(line[1]);
		ar.ReadString (line, 80);
		sscanf (line, "%i", &i);
		SetNumChannels(i); // allocate badChannels array
		ar.ReadString (line, 80);
		sscanf (line, "%i", &link);
		ar.ReadString (line, 80);
		sscanf (line, "%i", &numBad);
		for (i=0; i< numChannels; i++) badChannel[i]=FALSE;
		for (i=0; i< numBad; i++) {
			ar.ReadString (line, 80);
			sscanf (line, "%i", &j);
			badChannel[j]=TRUE;
		}
		ar.ReadString (line, 80);
		sscanf (line, "%i", &i);
		SetHitPolarity(i);
		biasVoltage = -1.0;
		if (version>1.201) {
			ar.ReadString (line, 80);
			sscanf (line, "%f", &biasVoltage);
		}
	}
}

void Module::SetBiasVoltage(float b)
{
	biasVoltage=b;
}

float Module::GetBiasVoltage()
{
	return biasVoltage;
}
